;**************************************************************************
;
; Filename : GEBUEHREN.ASM
; ------------------------
;
; (c) 1991 by Thomas Lehmann
;
;
; Jackophone V5.00
;
;
; Gebhren bzw. Gesprchsdauer Anzeige
;
;
;**************************************************************************

MODUL_START	set	*	; Gre des Moduls berechnen

; =========================================================================
;
; Funktion	: Gebhren bzw. Gesprchsdauer nachfhren
;
; Eingang	: zyklisch von Hauptprg. Schleife
;
; Ausgang	: ------
;
; Register	: Accu
;
; (c) by Thomas Lehmann Mai 1990
;
; =========================================================================

inc_geb	bbr GEB_OUT,hgeb_stat,inc_geb_end	; Anzeige ausgeschaltet
	bbs DIP_GEB,dip_stat,inc_gspr_tim	; keine Gebhrenauswertung

	; ==================
	; Gebhren errechnen
	; ==================

	bbr GEB_IMP,hgeb_stat,inc_geb_end	; keine Gebhren erkannt

	sei
	rmb GEB_IMP,hgeb_stat	; und neu auswerten
	cli

	ldx geb_cnt	; Anzahl erkannter Gebhrenimpulse low
	lda geb_cnt+1	; dito high
	jsr calc_geb_dm	; in DM umrechnen, DM:Accu, X,  Pf.:Y
	stx gesprd+1	; DM Betrag low
	sty gesprd	; Pfennige
	rts


	; =========================
	; Gesprchsdauer nachfhren
	; =========================

inc_gspr_tim lda uhr_buff+SEK	; aktuelle Sekunde laden
	cmp gesprd_sek	; ist Sekunde vorbei ?
	beq inc_geb_end	; nein, noch gleich, Ende

	sta gesprd_sek	; Sekunde eintragen

	lda gesprd	; Gesprchsdauer Sekunde laden
	inc a		; Sekunde nachfhren
	cmp #60		; berlauf ?
	bcc inc_gspr_1	; nein

	lda #00		; Sekunde Null laden
	inc gesprd+1	; Gesprchsdauer Minute nachfhren
inc_gspr_1	sta gesprd	; Gesprchsdauer low Byte nachfhren

inc_geb_end	rts



; =========================================================================
;
; Funktion	: Gesprchsdauer bzw. Gebhren ausgeben
;
; Eingang	: von Routine "Wahlablauf-Cursor aus"
;
; Ausgang	: -------
;
; Register	: Accu, X
;
; (c) by Thomas Lehmann Mai 1990
;
; =========================================================================

put_gesprd
	block
	bbr GEB_OUT,hgeb_stat,gesprd_end	; Anzeige ausgeschaltet
	bbs ST_BABY,no_ini_flag,gesprd_end	; Babyruf aktiv

	lda #18		; Cursorposition laden
	jsr curs_set	; Cursor setzen
	jsr space	; Trennung zwischen Rufnr. und Anzeige
	lda gesprd+1	; high Byte der Gesprchsdauer/Gebhren
	jsr putdez99	; und 2-stellig ausgeben

	lda #":"	; Trennzeichen fr Gesprchsdauer
	bbs DIP_GEB,dip_stat,gesprd_1	; keine Gebhrenauswertung

	lda #"."	; Trennzeichen fr Gebhren
gesprd_1	jsr putchar	; und ausgeben
	lda gesprd	; low Byte der Gesprchsdauer/Gebhren
	jmp putdez99	; und 2-stellig ausgeben

gesprd_end	rts
	bend


; =========================================================================
;
; Funktion	: DM Betrag fr Gebhreneinheiten * Faktor berrechnen
;
; Eingang	: Anzahl Gebhreneinheiten MSB Accu, LSB X
;
; Ausgang	: DM Betrag MSB Accu, LSB X, Pfennige in Y
;
; Register	: Accu, X, Y
;
; (c) by Thomas Lehmann Oktober 1991
;
; =========================================================================

calc_geb_dm	ldy #100
	jsr div16_8	; Einheiten / 100 == Betrag in DM, Rest in Y
	phy		; restliche Einheiten, max. 99 retten

	; DM Betrag fr Einheiten ohne Rest berrechnen
	; --------------------------------------------
	ldy geb_fakt	; Gebhrenfaktor laden, Multiplikant
	jsr mul16_8	; Wert in DM, (ACCU, X) * Y = (Accu, X)
	sta hlp_reg	; DM Betrag retten
	stx hlp_reg+1

	; Rest Betrag fr Einheiten in Pf. berrechnen
	; -------------------------------------------
	lda #$00	; high
	plx		; Restliche Einheiten
	jsr mul16_8	; Wert in Pfennigen, (ACCU, X) * Y = (Accu, X)

	; Rest Betrag in DM umrechnen
	; ---------------------------
	ldy #100
	jsr div16_8	; rest. Einh. / 100 == Betrag in DM, Pf. in Y
	phy		; Pfennige retten


	; beide DM Betrge addieren
	; -------------------------
	ldy hlp_reg+1	; low Byte des DM Betrags
	jsr add16_16	; Betrag + Rest DM (ACCU und X) + hlp_reg und Y

	ply		; Pfennige holen
	rts

	

; =========================================================================
;
; Funktion	: Gebhrenfaktor speichern
;
; Eingang	: Auswahl bei Speichern
;
; Ausgang	: Whltastatur auf Faktoreingabe umgeleitet
;
; Register	: X
;
; (c) by Thomas Lehmann Mai 1991
;
; =========================================================================

sp_gebfakt	ldx #WTA_SPGEB	; Prg. Nr. fr Whltaste laden
	stx wta_prg	; und bei Whltaste ausfhren

	lda geb_fakt	; aktuellen Gebhrenfaktor laden
	bra geb_out	; aktuellen Gebhrenfaktor ausgeben


; =========================================================================
;
; Funktion	: Gebhrenfaktor eingeben
;
; Eingang	: bei Whltaste und eingeschaltetem Gebhrenfaktor Modus
;	: Tastenwert (1 bis ...) im Accu
;
; Ausgang	: --------
;
; Register	: Accu
;
; (c) by Thomas Lehmann Mai 1991
;
; =========================================================================

wta_spgeb	jsr check_wta	; Taste im Gltigkeitsbereich ?
	bcs wta_spgeb_e	; nein, Ende

	jsr input	; Wert eintragen
	ldx input_po	; Offset auf akt. Eingabe
	cpx #2		; Wert komplett eingegeben ?
	bcc wta_spgeb_e	; nein

	ldx #00		; 2-stellige Eingabe holen (ACCU)
	jsr get_input
	sta geb_fakt	; Gebhrenfaktor eintragen

	; Gebhren Text ausgeben
	; ----------------------
geb_out	stz input_po	; Offset auf akt. Eingabe
	ldx #<geb_txt	; Text ausgeben
	ldy #>geb_txt
	jsr puttxt

	jsr putdez99	; Gebhrenfaktor dezimal ausgeben
	jmp space

wta_spgeb_e	rts


geb_txt	byte DSP_CLR|CURS_ON
	shift "Gebuehrenfaktor: "





; =========================================================================
;
; Funktion	: angefallene Gesprchsgebhren eintragen
;	  nach Trennen des Gesprchs
;
; Eingang	: ------
;
; Ausgang	: --------
;
; Register	: Accu, X, Y
;
; (c) by Thomas Lehmann Oktober 1991
;
; =========================================================================

geb_stat	ldx geb_cnt	; Anzahl Gebhrenimpulse des Gesprchs laden
	lda geb_cnt+1
	phx		; zur weiteren Addition retten
	pha

	; Gesamt Gebhrenimpulse eintragen
	; --------------------------------
	ldy struct_stat+STAT_GEB+1	; Gesamt Gebhren high
	sty hlp_reg			; fr add eintragen
	ldy struct_stat+STAT_GEB	; Gesamt Gebhren low
	
	jsr add16_16			; (ACCU und X) + (hlp_reg und Y)
	sta struct_stat+STAT_GEB+1	; Gesamt Gebhren high
	stx struct_stat+STAT_GEB	; Gesamt Gebhren low

	
	; Monats Gebhrenimpulse eintragen
	; --------------------------------
	ldx uhr_buff+MON			; aktuellen Monat laden (1-12)
	dex

	ldy struct_stat+STAT_GEB+1,x	; Monats Gebhren high
	sty hlp_reg			; fr add eintragen
	ldy struct_stat+STAT_GEB,x	; Monats Gebhren low
	pla				; Anzahl Gesprchs-Gebhrenimpulse
	plx
	jsr add16_16			; (ACCU und X) + (hlp_reg und Y)

	ldy uhr_buff+MON			; aktuellen Monat laden
	sta struct_stat+STAT_GEB+1,y	; Monats Gebhren high
	txa
	sta struct_stat+STAT_GEB,y	; Monats Gebhren low
	rts


; --------------------------------------------------------------------------

HLP	set	*
_GEBUEHREN	equ	HLP-MODUL_START		; Gre des Moduls

