;--------------	Snoopy V1.4 - written by Gerson Kurz 

		incpath	include:
		maclib	sm.mac
		macfile	libraries/startup.i
		macfile	macro.i
		include	snoopy.i

		DETACHSTARTUPARGS <arg.template>,<arg.first>,maincode,5000

;--------------	reset all data structures
maincode	bsr	SetupSnoopy
		tst	d0
		beq	.FAIL

;--------------	give out initial welcome to the user
		SHOWMSG	hello

;--------------	main loop ! Wait for signals to arrive, and process them 
.MainLoop	move.l	(waitsignals),d0
		CALL	Wait,<(execBase).w>
		move.l	d0,(signals)

;--------------	CTRL+C terminates the program
		BRBS	#SIGBREAKB_CTRL_C,d0,.DONE

;--------------	if we got a message, then answer it
		move.l	d0,d1
		and.l	(answersignal),d1
		tst.l	d1
		beq.b	.NoMessage
		bsr	HandleReply
		bra.b	.MainLoop
.NoMessage
;--------------	CTRL+D disables output
		BRBC.B	#SIGBREAKB_CTRL_D,d0,.DisableOutput
		tst.b	(outputactive)
		beq.b	.MainLoop
		move.b	#FALSE,(outputactive)
		SHOWMSG	disableout
		bra	.MainLoop
.DisableOutput
;--------------	CTRL+E disables output
		BRBC.B	#SIGBREAKB_CTRL_E,d0,.EnableOutput
		tst.b	(outputactive)
		bne.b	.MainLoop
		move.b	#TRUE,(outputactive)
		SHOWMSG	enableout
		bra	.MainLoop
.EnableOutput
;--------------	CTRL+F toggles the task info feature
		BRBC.B	#SIGBREAKB_CTRL_F,d0,.ToggleTI
		tst.l	(arg.taskinfo)
		bne.b	.TI_Disabled
		move.l	#TRUE,(arg.taskinfo)
		SHOWMSG	TIenabled
		bra	.MainLoop
.TI_Disabled	SHOWMSG	TIdisabled
		clr.l	(arg.taskinfo)
.ToggleTI
		bra	.MainLoop

.DONE		bsr	CleanupSnoopy
		moveq	#RETURN_OK,d0
		rts

.FAIL		bsr	CleanupSnoopy
		moveq	#RETURN_ERROR,d0
		rts

***********************************************************************************
;--------------	prepare Snoopy for action by opening all required resources and stuff
		ENTRY	SetupSnoopy,d1-d7/a0-a6

;--------------	setup list headers
		lea	(hides),a0
		NEWLIST	a0
		lea	(bases),a0
		NEWLIST	a0
		lea	(watches),a0
		NEWLIST	a0
		lea	(shows),a0
		NEWLIST	a0
		move.b	#TRUE,(outputactive)

;--------------	open icon.library
		lea	(iconName),a1
		moveq	#0,d0
		CALL	OpenLibrary,<(execBase).w>
		move.l	d0,(iconBase)

;--------------	NOT NEEDED ANYMORE, DETACHSTARTUPARGS TAKES CARE OF THIS
;--------------	open dos.library
;		lea	(dos.name),a1
;		moveq	#37,d0
;		CALL	OpenLibrary,<(execBase).w>
;		move.l	d0,(dosBase)
;		beq	.OPENDOSFAILED
;--------------	parse CLI arguments
;		move.l	#arg.template,d1
;		move.l	#arg.first,d2
;		moveq	#0,d3
;		CALL	ReadArgs,<(dosBase)>
;		move.l	d0,(arg.rdargs)
;		beq	.READARGSFAILED
		bsr	ReadWBArgs

;--------------	try to open script file
		move.l	(arg.script),d0
		bne.b	.CallParser
		move.l	#default.script,d0
.CallParser	movea.l	d0,a0
		bsr	ParseInputFile
		tst	d0
		beq	SetupSnoopy_done

;--------------	create msgport
		CALL	CreateMsgPort,<(execBase).w>
		move.l	d0,(myport)
		beq	.NOMSGPORT
		move.l	(ThisTask,a6),(thistask)
		movea.l	d0,a1
		move.l	#myport.name,(LN_NAME,a1)
		moveq	#0,d0
		move.b	(MP_SIGBIT,a1),d0
		moveq	#1,d1
		lsl.l	d0,d1
		move.l	d1,(answersignal)
		or.l	#MASK_SIGNALS,d1
		move.l	d1,(waitsignals)
		CALL	AddPort

;--------------	open CON: window
		move.l	#window.name,d1
		tst.l	(arg.output)
		beq.b	.DefaultWindow
		move.l	(arg.output),d1
.DefaultWindow	move.l	#MODE_READWRITE,d2
		CALL	Open,<(dosBase)>
		move.l	d0,(window)
		beq.b	.NOWINDOW

		bsr	PatchFunctions
		tst	d0
		beq	.PATCHFAILED

;--------------	perhaps we should make some SHOWs ?
		tst.l	(arg.show)
		beq.b	.NoSHOWs
		bsr	SetCmdlineSHOWs
.NoSHOWs
;--------------	set my priority (if applicable)
		move.l	(mypri),d0
		tst.l	(arg.pri)
		beq.b	.nopricmd
		movea.l	(arg.pri),a0
		move.l	(a0),d0
.nopricmd	movea.l	(execBase).w,a6
		movea.l	(ThisTask,a6),a1
		CALL	SetTaskPri

		moveq	#TRUE,d0
		bra.b	SetupSnoopy_done

.PATCHFAILED	lea	(nopatch.err),a0
		bra.b	.FAIL
.NOWINDOW	lea	(nowindow.err),a0
		bra.b	.FAIL		
.NOMSGPORT	lea	(nomsgport.err),a0
.FAIL		bsr	ShowErrorMessage
		bra.b	.OPENDOSFAILED

.READARGSFAILED	bsr	ShowDosError
.OPENDOSFAILED	moveq	#FALSE,d0
		DONE	SetupSnoopy
	
***********************************************************************************
;--------------	remove all open Snoopy resources
;--------------	
		ENTRY	CleanupSnoopy,d0-d7/a0-a6

		bsr	RemoveFunctions	; remove all functions
		bsr	RemoveBases	; remove all library bases

;--------------	close window
		move.l	(window),d1
		beq.b	.NoWindow
		CALL	Close,<(dosBase)>
.NoWindow
;--------------	remove msgport
		move.l	(myport),d0
		beq.b	.NoMsgPort
		movea.l	d0,a1
		CALL	RemPort,<(execBase).w>
		bsr	RemoveAllMessages
		movea.l	(myport),a0
		CALL	DeleteMsgPort
.NoMsgPort
;--------------	close CLI arguments
		move.l	(arg.rdargs),d1
		beq.b	.NoRdArgs
		CALL	FreeArgs,<(dosBase)>
.NoRdArgs
;--------------	NOT NEEDED ANYMORE, DETACHSTARTUPARGS TAKES CARE OF THIS
;--------------	close dos.library properly
;		move.l	(dosBase),d0
;		beq.b	.NoDosBase
;		movea.l	d0,a1
;		CALL	CloseLibrary,<(execBase).w>
;.NoDosBase	
;--------------	close icon.library properly
		move.l	(iconBase),d0
		beq.b	.NoIconBase
		movea.l	d0,a1
		CALL	CloseLibrary,<(execBase).w>
.NoIconBase	
;--------------	remove allocated lists
		lea	(bases),a0
		move.l	#lbi_SIZEOF,d0
		bsr.b	RemoveList
		lea	(watches),a0
		move.l	#pin_SIZEOF,d0
		bsr.b	RemoveList
		lea	(hides),a0
		move.l	#task_SIZEOF,d0
		bsr.b	RemoveList
		lea	(shows),a0
		move.l	#task_SIZEOF,d0
		bsr.b	RemoveList
		DONE	CleanupSnoopy

*******************************************************************************
;--------------	clears an allocated list with all its elements
;--------------
;--------------	=> a0: struct List *
;--------------	   d0: LONG size of each element (0=don't call FreeMem() on it)
;--------------
		ENTRY	RemoveList,d0-d7/a0-a6
		movea.l	a0,a5
		move.l	d0,d7
		movea.l	(execBase).w,a6
rl.LOOP		IFEMPTY	a5,rl.DONE
		move.l	(LH_HEAD,a5),a4
		movea.l	a5,a0
		REMHEAD	
		tst.l	d7
		beq.b	rl.LOOP
		movea.l	a4,a1
		move.l	d7,d0
		CALL	FreeMem
		bra.b	rl.LOOP
rl.DONE		DONE	RemoveList

***********************************************************************************
;--------------	show a DOS errorcode

		ENTRY	ShowDosError,d0-d7/a0-a6
		CALL	IoErr,<(dosBase)>
		move.l	d0,d1
		moveq	#0,d2
		CALL	PrintFault
		DONE	ShowDosError

***********************************************************************************
;--------------	yeah, we got ourselfs a message; now show it and go home
;--------------	
		ENTRY	HandleReply,d0-d7/a0-a6

.LOOP		movea.l	(myport),a0
		CALL	GetMsg,<(execBase).w>
		move.l	d0,(message)
		beq	HandleReply_done

;--------------	now we have the message and its data, so lets go for it !
		tst.b	(outputactive)
		beq.b	.SkipOutput
		movea.l	d0,a5			; a5=message

;--------------	if the task is hidden, skip output
		bsr	IsTaskHidden
		tst	d0
		bne.b	.SkipOutput

;--------------	show extended task info if applicable
		tst.l	(arg.taskinfo)
		beq.b	.NoTaskInfo
		bsr	ShowTaskInfo
		bsr.b	.NEWLINE
.NoTaskInfo
;--------------	prepare output data
		move.l	(pcm_Info,a5),a4	; a4=PatchInfo
		bsr	SetupOutputData

;--------------	send message
		lea.l	(pin_Template,a4),a0
		bsr	ShowMessage
		bsr.b	.NEWLINE
.SkipOutput
;--------------	free message memory
		move.l	(message),a1
		move.l	#pcm_SIZEOF,d0
		CALL	FreeMem,<(execBase).w>

		bra	.LOOP
.NEWLINE	movea.l	(dosBase),a6
		move.l	(window),d1
		move.l	#newline.msg,d2
		moveq	#2,d3
		CALL	Write
		move.l	(window),d1
		CALL	Flush
		rts
		DONE	HandleReply

***********************************************************************************
;--------------	checks if the current task is hidden, returns TRUE if so
		ENTRY	IsTaskHidden,d1-d7/a0-a6

;--------------	find taskname to a4
		movea.l	(pcm_Task,a5),a0
		cmpi.b	#NT_PROCESS,(LN_TYPE,a0)
		bne.b	.DEFAULT
		tst.l	(pr_TaskNum,a0)
		beq.b	.DEFAULT
		move.l	(pr_CLI,a0),d0
		lsl.l	#2,d0
		movea.l	d0,a0
		move.l	(cli_CommandName,a0),d0
		lsl.l	#2,d0
		addq.l	#1,d0
		movea.l	d0,a4
		bra.b	.DONE
.DEFAULT	move.l	(LN_NAME,a0),a4
.DONE	

;--------------	handle SHOWs
		tst.b	(showsactive)
		beq.b	.NoSHOWs
		lea	(shows),a5
		bsr.b	CheckIfIsInList
		not.l	d0
		bra.b	IsTaskHidden_done
.NoSHOWs
;--------------	handle HIDEs
		lea	(hides),a5
		bsr.b	CheckIfIsInList
		DONE	IsTaskHidden

***********************************************************************************
;--------------	checks if an entry can be found in a list
;--------------	
;-------------- -> a5: (struct List *) HEAD
;--------------	   a4: (STRPTR) name
;--------------	<- d0: TRUE=Member of this list, FALSE otherwise
;--------------	
		ENTRY	CheckIfIsInList,d1-d7/a0-a6
		cmpa.l	#0,a4
		beq	.NotInList
		IFEMPTY	a5,.NotInList
		movea.l	(LH_HEAD,a5),a5
.LOOP		tst.l	(LN_SUCC,a5)	; a5 = current node
		beq.b	.NotInList
		movea.l	a4,a0
		lea	(task_Name,a5),a1
.STRICMP	move.b	(a1)+,d0
		beq.b	.InList
		move.b	(a0)+,d1
		UCASE	d1
		cmp.b	d1,d0
		beq.b	.STRICMP
		movea.l	(LN_SUCC,a5),a5
		bra.b	.LOOP
.NotInList	moveq	#0,d0
		bra.b	CheckIfIsInList_done

.InList		moveq	#-1,d0
		DONE	CheckIfIsInList

***********************************************************************************
;--------------	show an information line about the task that called this function
;--------------	
		ENTRY	ShowTaskInfo,d1-d7/a0-a6
		lea	(taskinfo.msg),a1
		movea.l	(pcm_Task,a5),a0
		move.l	a0,(output.args)
		cmpi.b	#NT_PROCESS,(LN_TYPE,a0)
		bne.b	.DEFAULT
		tst.l	(pr_TaskNum,a0)
		beq.b	.DEFAULT
		move.l	(pr_CLI,a0),d0
		lsl.l	#2,d0
		movea.l	d0,a0
		move.l	(cli_CommandName,a0),(output.args+4)
		lea	(taskbcpl.msg),a1
		bra.b	.DONE
.DEFAULT	move.l	(LN_NAME,a0),(output.args+4)
.DONE		move.l	a1,a0
		bsr	ShowMessage
		DONE	ShowTaskInfo

***********************************************************************************
;--------------	remove all remaining messages from the port (so that we can be sure
;--------------	we have not lost any memory
;--------------	
		ENTRY	RemoveAllMessages,d0-d7/a0-a6

;--------------	free message memory
		movea.l	(execBase).w,a6
.LOOP		movea.l	(myport),a0
		CALL	GetMsg
		tst.l	d0
		beq.b	.DONE
		movea.l	d0,a1
		move.l	#pcm_SIZEOF,d0
		CALL	FreeMem
		bra.b	.LOOP
.DONE		DONE	RemoveAllMessages

***********************************************************************************
;--------------	setup output.args to contain the registers as specified by 
;--------------	the register template
;--------------	
;--------------	-> a4: Patch Info
;--------------	   a5: Register Data
		ENTRY	SetupOutputData,d0-d7/a0-a6
		lea	(output.args),a0	; a0 = current output offset
		lea.l	(pin_Arguments,a4),a1	; a1 = current position in argument template

;--------------	done ? then lets go !
.LOOP		move.b	(a1)+,d0
		cmpi.b	#ARG_DONE,d0
		beq	SetupOutputData_done

;--------------	set correct register data to a2
		lea	(pcm_RegsBeforeCall,a5),a2 ; a2 = regs before call ( default )
		BRBC.B	#ARGB_RESULT,d0,.BeforeCall
		lea	(pcm_RegsAfterCall,a5),a2 ; a2 = regs before call ( default )
		bclr	#ARGB_RESULT,d0
.BeforeCall
;--------------	set correct jump table to a3
		lea	(.JumpWORD),a3
		BRBC.B	#ARGB_LONG,d0,.UseWORD
		lea	(.JumpLONG),a3
		bclr	#ARGB_LONG,d0
.UseWORD
;--------------	now get the register data and save it
		andi.w	#%1111,d0
		lsl.w	#2,d0
		movea.l	(a3,d0.w),a3
		jsr	a3
		bra.b	.LOOP

.JumpLONG	dc.l	.SaveD0L
		dc.l	.SaveD1L
		dc.l	.SaveD2L
		dc.l	.SaveD3L
		dc.l	.SaveD4L
		dc.l	.SaveD5L
		dc.l	.SaveD6L
		dc.l	.SaveD7L
		dc.l	.SaveA0L
		dc.l	.SaveA1L
		dc.l	.SaveA2L
		dc.l	.SaveA3L
		dc.l	.SaveA4L
		dc.l	.SaveA5L
		dc.l	.SaveA5L
		dc.l	.SaveA5L

.JumpWORD	dc.l	.SaveD0W
		dc.l	.SaveD1W
		dc.l	.SaveD2W
		dc.l	.SaveD3W
		dc.l	.SaveD4W
		dc.l	.SaveD5W
		dc.l	.SaveD6W
		dc.l	.SaveD7W
		dc.l	.SaveA0W
		dc.l	.SaveA1W
		dc.l	.SaveA2W
		dc.l	.SaveA3W
		dc.l	.SaveA4W
		dc.l	.SaveA5W
		dc.l	.SaveA5W
		dc.l	.SaveA5W

SAVEREG		MACRO
.Save\1L	move.l	(pcr_\1,a2),(a0)+
		rts
.Save\1W	move.w	(pcr_\1,a2),(a0)+
		rts
		ENDM

		SAVEREG	A0
		SAVEREG	A1
		SAVEREG	A2
		SAVEREG	A3
		SAVEREG	A4
		SAVEREG	A5
		SAVEREG	D0
		SAVEREG	D1
		SAVEREG	D2
		SAVEREG	D3
		SAVEREG	D4
		SAVEREG	D5
		SAVEREG	D6
		SAVEREG	D7

		DONE	SetupOutputData

***********************************************************************************
;--------------	send message to window
;--------------	
;--------------	=> a0: APTR string
;--------------	 
		ENTRY	ShowMessage,d0-d7/a0-a6
		move.l	(window),d1
		move.l	a0,d2
		move.l	#output.args,d3
		CALL	VFPrintf,<(dosBase)>
		DONE	ShowMessage

***********************************************************************************
;--------------	send error message to the default output stream
;--------------	
;--------------	=> a0: APTR string
;--------------	 
		ENTRY	ShowErrorMessage,d0-d7/a0-a6
		move.l	a0,d1
		move.l	#output.args,d2
		CALL	VPrintf,<(dosBase)>
		DONE	ShowErrorMessage

***********************************************************************************
;--------------	patch all functions in the watch list
;--------------	
		ENTRY	PatchFunctions,d1-d7/a0-a6
		lea	(watches),a5
		IFEMPTY	a5,.DONE
		movea.l	(LH_HEAD,a5),a5
		movea.l	(execBase).w,a6
.LOOP		tst.l	(LN_SUCC,a5)	; a5 = current node
		beq.b	.DONE
		movea.l	a5,a0
		bsr.b	InstallPatch
		tst	d0
		beq.b	.FAIL		
		movea.l	(LN_SUCC,a5),a5
		bra.b	.LOOP

.FAIL		moveq	#FALSE,d0
		bra.b	PatchFunctions_done

.DONE		moveq	#TRUE,d0
		DONE	PatchFunctions

***********************************************************************************
;--------------	Patch a system function
;-------------- 
;--------------	=> a0: APTR Patchinfo 
;--------------	<= d0: LONG success
;--------------	
		ENTRY	InstallPatch,d1-d7/a0-a6
		movea.l	a0,a5
		move.l	a0,(PatchInfoStruct)

;--------------	allocate patch duplicate
		move.l	#Patch_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem,<(execBase).w>
		tst.l	d0
		beq.b	.FAIL
		move.l	d0,a4

;--------------	setup original vector
		movea.l	(pin_LibraryBase,a5),a1
		move.w	(pin_Offset,a5),d1
		addq.w	#2,d1
		move.l	(a1,d1.w),(OriginalVector0+2)
		move.l	(a1,d1.w),(OriginalVector1+2)
		move.l	(a1,d1.w),(Patch_START+pih_OriginalFunction)

;--------------	copy patch memory
		move.l	d0,a0
		lea	(Patch_START),a1
		move.l	#(Patch_SIZEOF/2)-1,d0
.LOOP		move.w	(a1)+,(a0)+
		dbra	d0,.LOOP

;--------------	install new function
		move.l	(pin_LibraryBase,a5),a1
		move.w	(pin_Offset,a5),d0
		ext.l	d0
		movea.l	d0,a0
		move.l	a4,d0
		addi.l	#pih_SIZEOF,d0
		CALL	SetFunction
		
		moveq	#TRUE,d0
		bra.b	InstallPatch_done
		
.FAIL		moveq	#FALSE,d0
		DONE	InstallPatch

***********************************************************************************
;--------------	remove the patch of all functions in the watch list
;--------------	
		ENTRY	RemoveFunctions,d0-d7/a0-a6
		lea	(watches),a5
		IFEMPTY	a5,.DONE
		movea.l	(LH_HEAD,a5),a5
		movea.l	(execBase).w,a6
.LOOP		tst.l	(LN_SUCC,a5)	; a5 = current node
		beq.b	.DONE
		movea.l	a5,a0
		bsr.b	RemovePatch
		movea.l	(LN_SUCC,a5),a5
		bra.b	.LOOP
.DONE		DONE	RemoveFunctions

***********************************************************************************
;--------------	Remove patch of a system function
;-------------- 
;--------------	=> a0: APTR Patchinfo 
;--------------	
		ENTRY	RemovePatch,d0-d7/a0-a6
		movea.l	a0,a5

;--------------	test if it is our patch
		movea.l	(pin_LibraryBase,a5),a1
		move.w	(pin_Offset,a5),d1
		addq.w	#2,d1
		move.l	(a1,d1.w),a1
		subi.l	#pih_SIZEOF,a1
		cmpi.l	#PATCH_IDSTRING,(pih_Ident,a1)
		bne.b	RemovePatch_done
		move.l	a1,a4			; a4 = patched memory

;--------------	restore original function
		move.l	(pin_LibraryBase,a5),a1
		move.w	(pin_Offset,a5),d0
		ext.l	d0
		movea.l	d0,a0
		move.l	(pih_OriginalFunction,a4),d0
		CALL	SetFunction,<(execBase).w>

;--------------	free patched memory
		move.l	a4,a1
		move.l	#Patch_SIZEOF,d0
		CALL	FreeMem
		DONE	RemovePatch

***********************************************************************************
;--------------	this is the patch-function that replaces any original code.
;--------------	Note that we have a little recursion-protector here that will
;--------------	prevents recursions that occur inside the original vector 
;--------------	just aslong as the PatchCode() is active

Patch_START	dc.b	PATCH_IDSTRING
		dc.l	0		; original offset (to restore function)
PatchInfoStruct	dc.l	0		; pointer to patchinfo for this thing
PatchCode	tas	(recursion)
		bne	OriginalVector1
		push	a6
		bsr.b	SetupReplyMsg
		push	a6
		move.l	(4,sp),a6
OriginalVector0	jsr	$ffff0000
		pop	a6
		bsr	SendReplyMsg
		pop	a6
		clr.w	(recursion)
		rts

***********************************************************************************
;--------------	sets up the replymsg given by the patch function
;--------------	
;--------------	=> d0-d7/a0-a5: Registers before function call
;--------------	<= a6: PatchCallMessage structure
SetupReplyMsg	
;--------------	allocate data structure
		pushm	d0-d7/a0-a5
		movea.l	(execBase).w,a6
		move.l	(thistask),d0
		cmp.l	(ThisTask,a6),d0
		beq	.SKIPTHIS
		move.l	#pcm_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem
		move.l	d0,a6
		popm	d0-d7/a0-a5

;--------------	fail if NULL; else save all registers
		cmpa.l	#0,a6
		beq.b	.FAIL
		SAVEREGS pcm_RegsBeforeCall
.FAIL		rts
.SKIPTHIS	suba.l	a6,a6
		popm	d0-d7/a0-a5
		rts

***********************************************************************************
;--------------	sends the reply message back
;--------------	
;--------------	=> d0-d7/a0-a5: Registers after function call
;--------------	
SendReplyMsg	
;--------------	fail if NULL; else save all registers
		cmpa.l	#0,a6
		beq	.FAIL
		SAVEREGS pcm_RegsAfterCall
		pushm	d0-d7/a0-a5

		movea.l	(execBase).w,a0
		move.l	(ThisTask,a0),(pcm_Task,a6)
		move.l	(PatchInfoStruct),(pcm_Info,a6)
		movea.l	a6,a5			; a5 = Data structure
		movea.l	a0,a6

;--------------	setup other message stuff
		move.b	#NT_MESSAGE,(LN_TYPE,a5)
		move.w	#pcm_SIZEOF,(MN_LENGTH,a5)
		move.l	#0,(MN_REPLYPORT,a5)
		move.l	(myport),a0		; port
		move.l	a5,a1			; message
		CALL	PutMsg

		popm	d0-d7/a0-a5
.FAIL		rts

OriginalVector1	jsr	$ffff000
		rts

		even
Patch_SIZEOF	equ	*-Patch_START

***********************************************************************************
;--------------	parse a Snoopy input file
;--------------	
;--------------	-> a0: APTR name
;--------------	<= d0: BOOL success
		ENTRY	ParseInputFile,d1-d7/a0-a6

;--------------	open file
		move.l	a0,d1
		move.l	d1,(output.args)
		move.l	#MODE_OLDFILE,d2
		CALL	Open,<(dosBase)>
		move.l	d0,(filehandle)
		beq.b	.NoFile

;--------------	read all lines one-by-one
.LOOP		move.l	(filehandle),d1
		move.l	#input,d2
		move.l	#256,d3
		CALL	FGets
		tst.l	d0
		beq.b	.DONE
		bsr	ProcessInput
		tst.l	d0
		beq.b	.FAIL
		bra.b	.LOOP

;--------------	FGets() returned NULL; so we have to check if it was EOF or an error
.DONE		CALL	IoErr
		move.l	d0,d1
		beq.b	.ReallyDone

;--------------	it WAS an error, so show it and exit
		moveq	#0,d2
		CALL	PrintFault
.FAIL		move.l	(filehandle),d1
		CALL	Close
		bra.b	.QUIT

;--------------	return to sender
.ReallyDone	move.l	(filehandle),d1
		CALL	Close
		moveq	#TRUE,d0
		bra.b	ParseInputFile_done

;--------------	some error message
.NoFile		lea	(openfile.err),a0
		bsr	ShowErrorMessage
.QUIT		moveq	#FALSE,d0
		DONE	ParseInputFile


***********************************************************************************
;--------------	process current input data line (in 'input' buffer)
;--------------	
;--------------	<= d0: BOOL success
;--------------	
		ENTRY	ProcessInput,d1-d7/a0-a6

;--------------	remove comments and NEWLINE
		lea	(input),a0
		cmpi.b	#COMMENT_LINE,(a0)	; skip if its a comment line
		beq.b	.DONE
.CommentLoop	move.b	(a0)+,d0
		beq.b	.CommentDone		
		cmpi.b	#COMMENT,d0
		beq.b	.CommentFound
		cmpi.b	#"\n",d0
		bne.b	.CommentLoop	
.CommentFound	clr.b	-(a0)
.CommentDone
;--------------	skip the line if it only consists of whitespaces
		lea	(input),a0
.WhitespaceLoop	move.b	(a0)+,d0
		beq.b	.DONE
		cmpi.b	#" ",d0
		beq.b	.WhitespaceLoop
		cmpi.b	#"	",d0
		beq.b	.WhitespaceLoop
.NotWhitespaces
;--------------	see if it is one of the codewords we need 
		lea	(CodewordTable),a0
.CodewordLoop	tst.l	(a0)
		beq.b	.InvalidLine

;--------------	compare codeword and inputline
		move.l	(a0),a1
		lea	(input),a2
.STRICMP	move.b	(a1)+,d0
		beq.b	.Found
		move.b	(a2)+,d1
		UCASE	d1
		cmp.b	d1,d0
		beq.b	.STRICMP

.NotFound	addq.l	#8,a0
		bra.b	.CodewordLoop		

;--------------	codeword found -> call function !
.Found		tst.l	(4,a0)
		beq.b	.DONE
		move.l	(4,a0),a2
		jsr	a2
		bra.b	ProcessInput_done

.DONE		moveq	#TRUE,d0
		bra.b	ProcessInput_done

;--------------	invalid input line
.InvalidLine	move.l	#input,output.args
		lea	(invalidline.err),a0
		bsr	ShowErrorMessage
.QUIT		moveq	#FALSE,d0
		DONE	ProcessInput

***********************************************************************************
;--------------	setup a library base. the input line syntax is :
;--------------	
;--------------	base=<alias>,<complete name>[,<version>]
;--------------	
		ENTRY	SetBase,d1-d7/a0-a6

;--------------	find start of information
		lea	(input),a0
.FindStart	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#"=",d0
		bne.b	.FindStart

;--------------	load a new base to a5
		bsr	NewBase
		tst	d0
		beq	SetBase_done

;--------------	copy base name
		lea	(lbi_BaseName,a5),a1
		moveq	#MAX_BASENAME-2,d1
.CopyBaseName	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#",",d0
		beq.b	.BaseNameDone
		UCASE	d0
		move.b	d0,(a1)+
		dbra	d1,.CopyBaseName
.FindComma	move.b	(a0)+,d0
		beq.b	.InvalidLine
		cmpi.b	#",",d0
		bne.b	.FindComma
.BaseNameDone	clr.b	(a1)+
		move.l	a0,d7		; d7 = library name

;--------------	strip trailing whitespaces
.FindEnd	move.b	(a0)+,d0
		beq.b	.EndFound
		cmpi.b	#",",d0
		bne.b	.FindEnd
.Version	clr.b	(-1,a0)
		moveq	#STVFORMAT_DEC,d0
		bsr	StringToValue
		move.l	d0,(lbi_Version,a5)	; , library version
		bra.b	.GoOn

.EndFound	subq.l	#1,a0
.WhileIsWhite	cmpa.l	d7,a0
		beq.b	.WhiteIsDone
		move.b	-(a0),d0
		cmpi.b	#" ",d0
		beq.b	.WhileIsWhite
		cmpi.b	#"	",d0
		beq.b	.WhileIsWhite
.WhiteIsDone	clr.b	(1,a0)
.GoOn
;--------------	now try to open the library
		movea.l	d7,a1
		move.l	(lbi_Version,a5),d0
		CALL	OpenLibrary,<(execBase).w>
		move.l	d0,(lbi_Base,a5)
		beq.b	.CannotOpenLib

.DONE		moveq	#TRUE,d0
		bra.b	SetBase_done

;--------------	cannot open specified library
.CannotOpenLib	move.l	d7,output.args
		lea	(openbase.err),a0
		bra.b	.SHOWERROR

;--------------	invalid input line
.InvalidLine	move.l	#input,output.args
		lea	(invalidline.err),a0
.SHOWERROR	bsr	ShowErrorMessage
.QUIT		moveq	#FALSE,d0
		DONE	SetBase

***********************************************************************************
;--------------	load new base to a5
;--------------	
;--------------	<= a5: APTR Base
;--------------	   d0: BOOL success
		ENTRY	NewBase,d1-d7/a0-a4/a6
		move.l	#lbi_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem,<(execBase).w>
		tst.l	d0
		beq.b	NewBase_done
		movea.l	d0,a5
		movea.l	d0,a1
		lea	(bases),a0
		ADDHEAD
		DONE	NewBase

***********************************************************************************
;--------------	setup a watch slot. the input line syntax is :
;--------------	
;--------------	watch=<base>,<offset>,<regs>,<template>
;--------------	
		ENTRY	SetWatch,d1-d7/a0-a6

;--------------	find start of information
		lea	(input),a0
.FindStart	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#"=",d0
		bne.b	.FindStart
		move.l	a0,d7

;--------------	load a new watch to a5
		bsr	NewWatch
		tst	d0
		beq	SetWatch_done

;--------------	try to find a base for this watch
		lea	(bases),a1
		IFEMPTY	a1,.NoBaseFound
		movea.l	(LH_HEAD,a1),a1
.FindBaseLOOP	tst.l	(LN_SUCC,a1)	; a1 = current node
		beq.b	.NoBaseFound
		lea	(lbi_BaseName,a1),a2
		movea.l	d7,a0
.STRICMP	move.b	(a0)+,d0
		UCASE	d0
		move.b	(a2)+,d1
		beq.b	.BaseFound
		cmp.b	d1,d0
		beq.b	.STRICMP
		movea.l	(LN_SUCC,a1),a1
		bra.b	.FindBaseLOOP
.BaseFound	move.l	(lbi_Base,a1),(pin_LibraryBase,a5)

;--------------	find Offset
		movea.l	d7,a0
.FindFirstComma	move.b	(a0)+,d0
		beq.b	.InvalidLine
		cmpi.b	#",",d0
		bne.b	.FindFirstComma
		moveq	#STVFORMAT_DEC,d0
		bsr	StringToValue
		move.w	d0,(pin_Offset,a5)

;--------------	now parse the register information
		bsr	ParseRegisterInfo
		tst	d0
		beq.b	.InvalidLine

;--------------	the rest of this line is interpreted as the argument template
		lea	(pin_Template,a5),a1
		moveq	#MAX_TEMPLATE-2,d0
.CopyTemplate	move.b	(a0)+,(a1)+
		beq.b	.TemplateDone
		dbra	d0,.CopyTemplate
		clr.b	(a1)+
.TemplateDone

.DONE		moveq	#TRUE,d0
		bra.b	SetWatch_done

;--------------	cannot find base an error
.NoBaseFound	lea	(nobasefnd.err),a0
		bra.b	.SHOWERROR

;--------------	invalid input line
.InvalidLine	lea	(invalidline.err),a0
.SHOWERROR	move.l	#input,output.args
		bsr	ShowErrorMessage
.QUIT		moveq	#FALSE,d0
		DONE	SetWatch

***********************************************************************************
;--------------	load new watch to a5
;--------------	
;--------------	<= a5: APTR Watch
;--------------	   d0: BOOL success
		ENTRY	NewWatch,d1-d7/a0-a4/a6
		move.l	#pin_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem,<(execBase).w>
		tst.l	d0
		beq.b	NewWatch_done
		movea.l	d0,a5
		movea.l	d0,a1
		lea	(watches),a0
		ADDHEAD
		DONE	NewWatch

***********************************************************************************
;--------------	read in register information stored in a syntax of
;--------------	
;--------------	 <register 0>/<register 1>/.../<register n>
;--------------	
;--------------	where <register x> is
;--------------	
;--------------		[R](D|A)(0..7)[L]
;--------------	
;--------------	=> A0: APTR start of register information
;--------------	   A5: struct PatchInfo *pi
;--------------	

		ENTRY	ParseRegisterInfo,d1-d7/a1-a6

		lea	(pin_Arguments,a5),a4	; load register data in a4
.LOOP		moveq	#0,d0
.GetRegister	move.b	(a0)+,d1
		beq	.InvalidLine
		UCASE	d1

;--------------	a leading R signals that this register is a return value
		cmpi.b	#"R",d1
		bne.b	.NotReturn
		bset	#ARGB_RESULT,d0
		bra.b	.GetRegister
.NotReturn	
;--------------	a D is for data registers
		cmpi.b	#"D",d1
		bne.b	.NotData
		move.b	(a0)+,d1
		beq	.InvalidLine
		subi.b	#"0",d1
		andi.b	#%111,d1
		or.b	d1,d0
		bra.b	.GetRegister

;--------------	an A stands for address registers
.NotData	cmpi.b	#"A",d1
		bne.b	.NotAddr
		move.b	(a0)+,d1
		beq.b	.InvalidLine
		subi.b	#"0",d1
		andi.b	#%111,d1
		or.b	d1,d0
		ori.b	#ARG_A0,d0
		bra.b	.GetRegister

;--------------	an I stands for address registers relative
.NotAddr	cmpi.b	#"I",d1
		bne.b	.NotRelative
		move.b	(a0)+,d1
		beq.b	.InvalidLine
		subi.b	#"0",d1
		andi.b	#%111,d1
		or.b	d1,d0
		ori.b	#ARG_I0,d0
		bra.b	.GetRegister
.NotRelative
;--------------	a trailing L stands for LONG (as opposed to WORD=defaults)
		cmpi.b	#"L",d1
		bne.b	.NotLong
		bset	#ARGB_LONG,d0
		bra	.GetRegister
.NotLong
;--------------	a slash / indicates the end of one register definition, so position
;--------------	to the next register slot
		cmpi.b	#"/",d1
		bne.b	.NotSlash
		move.b	d0,(a4)+
		bra	.LOOP
.NotSlash
;--------------	a comma , indicates the end of the register list definition
		cmpi.b	#",",d1
		bne.b	.InvalidLine
		move.b	d0,(a4)+
		move.b	#ARG_DONE,(a4)
		moveq	#TRUE,d0
		bra.b	ParseRegisterInfo_done

.InvalidLine	moveq	#FALSE,d0
		DONE	ParseRegisterInfo

*******************************************************************************
;--------------	Diese Funktion konvertiert einen String in eine Zahl, wobei der
;--------------	String in Hexadezimal($),Dezimal(#),Oktal(@) oder Dual(%) vorliegen
;--------------	kann. Zustzlich kann eine Voreinstellung angenommen werden, um
;--------------	z.B. stndige '#' bei rein Dezimalen Strings zu berbrcken.
;--------------	Irgendwann mal in absolut ferner Zukunft habe ich vor, hier noch
;--------------	einen 'Calculator' reinzuhngen: Damit wre in jedem Programm,
;--------------	da ber StringToValue aus Strings Zahlen macht, gleich eine
;--------------	Taschenrechnerfunktion drinnen..
;--------------	=> d0: ULONG Vordefinierter Typ des Strings
;--------------	   a0: APTR Der String mit der Zahl
;--------------	<= d0: LONG Die (vorzeichenbehaftete) IntegerZahl

		ENTRY	StringToValue,d1-d2
		moveq	#10,d2
		cmp.l	#STVFORMAT_HEX,d0
		bne.b	.NotHex
		moveq	#16,d2
.NotHex		cmp.l	#STVFORMAT_BIN,d0
		bne.b	.NotBin
		moveq	#2,d2
.NotBin		cmp.l	#STVFORMAT_OCT,d0
		bne.b	.NotOct
		moveq	#8,d2
.NotOct		cmpi.b	#'-',(a0)
		bne.b	.NormalUse
		addq.l	#1,a0
		bsr.b	.Check4Sign
		bsr.b	.Get_s2v
		neg.l	d0
		bra	StringToValue_done

.NormalUse	bsr.b	.Check4Sign
		bsr.b	.Get_s2v
		bra	StringToValue_done

.Check4Sign	move.b	(a0),d0
		cmpi.b	#STVTYPE_DEC,d0
		bne.b	1$
		moveq	#10,d2
		addq.l	#1,a0
1$		cmpi.b	#STVTYPE_HEX,d0
		bne.b	2$
		moveq	#16,d2
		addq.l	#1,a0
2$		cmpi.b	#STVTYPE_OCT,d0
		bne.b	3$
		moveq	#8,d2
		addq.l	#1,a0
3$		cmpi.b	#STVTYPE_BIN,d0
		bne.b	4$
		moveq	#2,d2
		addq.l	#1,a0
4$		moveq	#0,d0
		rts

.Get_s2v	moveq	#0,d1
		move.b	(a0)+,d1
		cmpi.b	#'a',d1
		blt.b	.NoUpcaseChose
		cmpi.b	#'z',d1
		bgt.b	.NoUpcaseChose
		sub.b	#('a'-'A'),d1 
.NoUpcaseChose 	subi.b	#"0",d1
		cmpi.b	#10,d1
		bls.b	.s2v_Overdose
		cmpi.l	#16,d2
		bne.s	.s2v_Overdose
		subq.b	#'A'-('9'+1),d1
.s2v_Overdose	cmp.b	d2,d1
		bcc.b	.s2v_Done
		bsr.b	ExtendedMulu
		add	d1,d0
		bra.b	.Get_s2v
.s2v_Done	rts
		DONE	StringToValue

;--------------	Diese Funktion Multipliziert zwei Langwerte
;--------------	=> d1: LONG Wert 1 
;--------------	   d2: LONG Wert 2
;--------------	<= d0: LONG Result
		ENTRY	ExtendedMulu,d1/d2
		cmpi.l	#$7FFF,d2
		bcc.b	2$
		cmpi.l	#$7FFF,d0
		bcc.b	2$
		mulu	d2,d0
		bra.b	ExtendedMulu_done
2$		moveq	#0,d1
		subq.l	#1,d2
1$		add.l	d0,d1
		dbra	d2,1$
		move.l	d1,d0
		DONE	ExtendedMulu

***********************************************************************************
;--------------	remove all library bases opened by the Snoopy parser
		ENTRY	RemoveBases,d0-d7/a0-a6
	
;--------------	loop through all bases and close those librarys
		lea	(bases),a5
		IFEMPTY	a5,.DONE
		movea.l	(LH_HEAD,a5),a5
		movea.l	(execBase).w,a6
.LOOP		tst.l	(LN_SUCC,a5)	; a5 = current node
		beq.b	.DONE
		move.l	(lbi_Base,a5),d0
		beq.b	.NEXT
		clr.l	(lbi_Base,a5)
		movea.l	d0,a1
		CALL	CloseLibrary
.NEXT		movea.l	(LN_SUCC,a5),a5
		bra.b	.LOOP
.DONE		DONE	RemoveBases


***********************************************************************************
;--------------	hide a task from Snoopy. the syntax is
;--------------	
;--------------	hide=<taskname>
;--------------	
		ENTRY	SetHide,d1-d7/a0-a6

;--------------	find start of information
		lea	(input),a0
.FindStart	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#"=",d0
		bne.b	.FindStart
		move.l	a0,d7

;--------------	load a new watch to a5
		bsr	NewHide
		tst	d0
		beq	SetHide_done

;--------------	copy name of HIDE list
		lea	(task_Name,a5),a1
		moveq	#MAX_HIDENAME-2,d0
.COPY		move.b	(a0)+,d1
		beq.b	.DONE
		UCASE	d1
		move.b	d1,(a1)+
		dbra	d0,.COPY
.DONE		clr.b	(a1)+
		moveq	#TRUE,d0
		bra.b	SetHide_done

;--------------	invalid input line
.InvalidLine	lea	(invalidline.err),a0
		move.l	#input,output.args
		bsr	ShowErrorMessage
		moveq	#FALSE,d0
		DONE	SetHide

***********************************************************************************
;--------------	load new hide to a5
;--------------	
;--------------	<= a5: APTR hide
;--------------	   d0: BOOL success
		ENTRY	NewHide,d1-d7/a0-a4/a6
		move.l	#task_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem,<(execBase).w>
		tst.l	d0
		beq.b	NewHide_done
		movea.l	d0,a5
		movea.l	d0,a1
		lea	(hides),a0
		ADDHEAD
		DONE	NewHide

***********************************************************************************
;--------------	show a task from Snoopy. the syntax is
;--------------	
;--------------	show=<taskname>
;--------------	
		ENTRY	SetShow,d1-d7/a0-a6

;--------------	find start of information
		lea	(input),a0
.FindStart	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#"=",d0
		bne.b	.FindStart
		move.l	a0,d7

;--------------	load a new watch to a5
		bsr	NewShow
		tst	d0
		beq	SetShow_done

		bsr.b	CopySHOWName
		moveq	#TRUE,d0
		bra.b	SetShow_done

;--------------	invalid input line
.InvalidLine	lea	(invalidline.err),a0
		move.l	#input,output.args
		bsr	ShowErrorMessage
		moveq	#FALSE,d0
		DONE	SetShow

;--------------	copy name of SHOW list
CopySHOWName	lea	(task_Name,a5),a1
		moveq	#MAX_HIDENAME-2,d0
.COPY		move.b	(a0)+,d1
		beq.b	.DONE
		UCASE	d1
		move.b	d1,(a1)+
		dbra	d0,.COPY
.DONE		clr.b	(a1)+
		move.b	#-1,(showsactive)
		rts

***********************************************************************************
;--------------	add SHOWs from the commandline
;--------------	
		ENTRY	SetCmdlineSHOWs,d1-d7/a0-a6
		move.l	(arg.show),d0
		beq.b	.DONE
		movea.l	d0,a4
.LOOP		move.l	(a4)+,d1
		beq.b	.DONE
		bsr.b	NewShow
		tst	d0
		beq.b	SetCmdlineSHOWs_done
		movea.l	d1,a0
		bsr	CopySHOWName
		bra.b	.LOOP
.DONE		moveq	#TRUE,d0
		DONE	SetCmdlineSHOWs

***********************************************************************************
;--------------	load new SHOW to a5
;--------------	
;--------------	<= a5: APTR hide
;--------------	   d0: BOOL success
		ENTRY	NewShow,d1-d7/a0-a4/a6
		move.l	#task_SIZEOF,d0
		move.l	#MEMF_PUBLIC|MEMF_CLEAR,d1
		CALL	AllocMem,<(execBase).w>
		tst.l	d0
		beq.b	NewShow_done
		movea.l	d0,a5
		movea.l	d0,a1
		lea	(shows),a0
		ADDHEAD
		DONE	NewShow

***********************************************************************************
;--------------	set snoopy task priority. The syntax is
;--------------	
;--------------	pri=<level>
;--------------	
;--------------	level should be in the range of -5 to +5 
;--------------	
		ENTRY	SetPri,d1-d7/a0-a6

;--------------	find start of information
		lea	(input),a0
.FindStart	move.b	(a0)+,d0
		beq	.InvalidLine
		cmpi.b	#"=",d0
		bne.b	.FindStart
		move.l	a0,d7

		moveq	#STVFORMAT_DEC,d0
		bsr	StringToValue
		move.l	d0,(mypri)

		moveq	#TRUE,d0
		bra.b	SetPri_done

;--------------	invalid input line
.InvalidLine	lea	(invalidline.err),a0
		move.l	#input,output.args
		bsr	ShowErrorMessage
		moveq	#FALSE,d0
		DONE	SetPri


***********************************************************************************
		ENTRY	ReadWBArgs,d0-d7/a0-a6
		lea	(TTHStruktur),a3
		move.l	(tth_WBStartup,a3),d0		; WBenchMsg vorhanden?
		beq	ReadWBArgs_done
		tst.l	(arg.rdargs)
		bne	ReadWBArgs_done
		clr.l	(arg.rdargs)

;--------------	normaler Ablauf von Workbench
.vonWorkbench	move.l	d0,a3
		move.l	(sm_NumArgs,a3),d0
		move.l	(sm_ArgList,a3),a3
		cmpi.l	#1,d0
		beq.b	.DEFSTART
		lea	(wa_SIZEOF,a3),a3

.DEFSTART	move.l	(wa_Lock,a3),d1			; DirLock zeigt auf
		CALL	CurrentDir,<(dosBase)>		; Pfad des Programms
		move.l	(wa_Name,a3),a0
.LeseIcon	CALL	GetDiskObject,<(iconBase)>	; ".info"-Datei lesen
		BRF	d0,ReadWBArgs_done		; Fehler, dann weiter
		move.l	d0,(diskObject)	

;--------------	Tool Types auswerten
		move.l	d0,a3
		cmpi.b	#WBPROJECT,(do_PAD_BYTE-1,a3)
		beq	.USEPROJECT
		move.l	(do_ToolTypes,a3),a3	
		move.l	a3,a0
		lea	(TTScript),a1
		push	a0
		CALL	FindToolType
		pop	a0
		move.l	d0,(arg.script)
		lea	(TTOutput),a1
		push	a0
		CALL	FindToolType
		pop	a0
		move.l	d0,(arg.output)
		lea	(TTPri),a1
		push	a0
		CALL	FindToolType
		pop	a0
		tst.l	d0
		beq.b	.SKIPPRI
		move.l	#TTPri.value,(arg.pri)
		movea.l	d0,a0
		moveq	#STVFORMAT_DEC,d0
		bsr	StringToValue
		move.l	d0,(TTPri.value)
.SKIPPRI	bra.b	ReadWBArgs_done

.USEPROJECT	lea	(TTHStruktur),a3
		move.l	(tth_WBStartup,a3),a3
		move.l	(sm_ArgList,a3),a3
		lea	(wa_SIZEOF,a3),a3
		move.l	(wa_Name,a3),(arg.script)
		DONE	ReadWBArgs

***********************************************************************************
;--------------	data area

CodewordTable	dc.l	.Base,SetBase
		dc.l	.Watch,SetWatch
		dc.l	.Hide,SetHide
		dc.l	.Pri,SetPri
		dc.l	.Show,SetShow
		dc.l	0

.Base		dc.b	"BASE",0
.Watch		dc.b	"WATCH",0
.Hide		dc.b	"HIDE",0
.Pri		dc.b	"PRI",0
.Show		dc.b	"SHOW",0
		even
		dc.b	0,"$VER:",IDSTRING,0

TIenabled.msg	dc.b	'TaskInfo enabled\n',0
TIdisabled.msg	dc.b	'TaskInfo disabled\n',0
taskinfo.msg	dc.b	'Task %08lx : "%s"',0
taskbcpl.msg	dc.b	'Task %08lx : "%b"',0
hello.msg	dc.b	"Welcome to [1m",IDSTRING,"[0m - written by ",AUTHORSTRING,"\n",0
disableout.msg	dc.b	"[3mOutput disabled - press CTRL+E to enable again. [0m\n",0
enableout.msg	dc.b	"[3mOutput enabled - press CTRL+D to disable. [0m\n",0
window.name	dc.b	"con:0/0/640/150/",IDSTRING,0
dosName		dc.b	"dos.library",0
iconName	dc.b	"icon.library",0
taskName	dc.b	"Snoopy likes ULTRAWORLD",0
myport.name	dc.b	PROGRAMSTRING,0
arg.template	dc.b	"SCRIPT,"
		dc.b	"TASKINFO/S,"
		dc.b	"OUTPUT/K,"
		dc.b	"PRI/N,"
		dc.b	"SHOW/K/M"
		dc.b	0
TTScript	dc.b	"SCRIPT",0
TTOutput	dc.b	"OUTPUT",0
TTPri		dc.b	"PRI",0
default.script	dc.b	DEFAULTSCRIPT,0
newline.msg	dc.b	"\n",0

;--------------	error messages
openbase.err	dc.b	"Cannot open base for %s\n",0
openfile.err	dc.b	"Cannot open input file %s\n",0
invalidline.err	dc.b	"Invalid input line : %s\n",0
nobasefnd.err	dc.b	"Cannot find the base specified as %s\n",0
nowindow.err	dc.b	"Cannot open output stream\n",0
nomsgport.err	dc.b	"Cannot open Snoopy message port\n",0
nopatch.err	dc.b	"Cannot patch all of the functions in the script file, aborting\n",0
		even

		section	memory,bss

BSS_SECTION
TTPri.value	ds.l	1
diskObject	ds.l	1
recursion	ds.w	1	; if bit15 is 0, then no call is currently active 
mypri		ds.l	1
thistask	ds.l	1	; pointer to this task 
iconBase	ds.l	1
dosBase		ds.l	1	; pointer to dos.library base
filehandle	ds.l	1	; handle for the input script file
hides		ds.b	LH_SIZE	; list header for all hidden tasks
bases		ds.b	LH_SIZE	; list header for all library bases
watches		ds.b	LH_SIZE	; list header for all function watches
shows		ds.b	LH_SIZE	; list header for all shows 
input		ds.b	256	; current input line from script file
arg.rdargs	ds.l	1	; structure used by ReadArgs()
arg.first	
arg.script	ds.l	1	; name of scriptfile or NULL for defaults
arg.taskinfo	ds.l	1	; TRUE if extended task info is to be given
arg.output	ds.l	1	; name of redirected filename or NULL for the window
arg.pri		ds.l	1	; priority number or NULL
arg.show	ds.l	1	; array of tasknames to show or NULL
message		ds.l	1	; last message recieved by GetMsg()
myport		ds.l	1	; my global answerport
answersignal	ds.l	1	; the signalmask used for SendMsg()/GetMsg() answers
waitsignals	ds.l	1	; signals to Wait() for
signals		ds.l	1	; signals actually recieved by Wait()
window		ds.l	1	; output handle of CON: window
output.args	ds.l	MAX_OUTPUTARGS
outputactive	ds.b	1
showsactive	ds.b	1
		even
BSS_SIZEOF	equ	*-BSS_SECTION

		end

***********************************************************************************
;--------------	
