RCS_ID_C="$Id: amiga_generic.c,v 1.60 1993/10/16 17:20:49 too Exp $";
/*
 * Copyright (c) 1993 AmiTCP/IP Group, <amitcp-group@hut.fi>
 *                    Helsinki University of Technology, Finland.
 *                    All rights reserved.
 * 
 * Created: Fri Feb 12 13:26:55 1993 too
 * Last modified: Sat Oct 16 19:18:38 1993 too
 * 
 * HISTORY
 * $Log: amiga_generic.c,v $
 * Revision 1.60  1993/10/16  17:20:49  too
 * Fixed copying bug in SetDTableSize
 *
 * Revision 1.59  1993/09/14  22:15:16  jraja
 * Added zeroing of obits ib n WaitSelect() if new memory is allocated for it.
 *
 * Revision 1.58  1993/08/01  22:15:07  ppessi
 * Changed CloseSocket() to use api/apicall.h protos.
 *
 * Revision 1.57  1993/07/13  21:58:59  ppessi
 * Added internal prototypes; changed interface ioctl codes to 'I'
 *
 * Revision 1.56  1993/06/13  16:38:43  too
 * Added range check for fd2 in Dup2Socket()
 *
 * Revision 1.55  1993/06/11  20:18:09  too
 * Added Dup2Socket(). Chahanged ioctl to IoctlSocket()
 * Made function countSockets() which is needed to determine if last reference
 * to that socket is given away.
 *
 * Revision 1.54  1993/06/07  12:37:20  too
 * Changed inet_ntoa, netdatabase functions and WaitSelect() use
 * separate buffers for their dynamic buffers
 *
 * Revision 1.53  1993/06/04  11:16:15  jraja
 * Fixes for first public release.
 *
 * Revision 1.52  1993/06/03  20:20:54  too
 * The amount of selitems needed is now calculated during first selscan call
 *
 * Revision 1.51  93/06/03  00:29:06  00:29:06  jraja (Jarno Tapio Rajahalme)
 * Fixed obits start address calculation.
 * Fixed bcopy() on _SetDTableSize().
 * 
 * Revision 1.50  1993/06/02  10:42:55  too
 * Changed WaitSelect to use different timeout handling
 * And now WaitSelect handler fd_sets of variable size (fd_mask)
 *
 * Revision 1.49  1993/05/30  08:46:04  too
 * Now handles arrival of signal differently than timeout.
 * allocation of selbuf (now newselbuf) now dynamic and drom heap instead
 * of stack
 *
 * Revision 1.48  1993/05/27  21:08:26  ppessi
 * Fixed signal mask bug.
 *
 * Revision 1.47  1993/05/24  19:28:56  ppessi
 * Changed signal handling mechanism in tsleep_main() and WaitSelect()
 *
 * Revision 1.46  1993/05/23  15:07:14  too
 * Fixed *sigmp disapperance when calling tsleep_main multiple times
 * during one WaitSelect(9 call
 *
 * Revision 1.45  1993/05/23  14:33:40  too
 * Fixed setting of *sigmp when tsleep_main is not called, in waitselect
 *
 * Revision 1.44  1993/05/22  16:54:56  too
 * Fixed bug when itimerfix was called w/ NULL argument
 *
 * Revision 1.43  1993/05/17  01:07:47  ppessi
 * Changed RCS version.
 *
 * Revision 1.42  1993/05/15  10:07:20  too
 * Uses API_ENTRY() and API_EXIT() no more. Uses CHECK_TASK(),
 * ObtainSyscallSemaphore() -- ReleaseSyscallSemaphore(), API_STD_RETURN()
 * instead.
 *
 * Revision 1.41  93/05/05  16:09:34  16:09:34  puhuri (Markus Peuhkuri)
 * Fixes for finel demo.
 * 
 * Revision 1.40  93/05/04  14:07:31  14:07:31  too (Tomi Ollila)
 * Changed local api function names to the same as the names given
 * to API programmer, but w/ starting _
 * 
 * Revision 1.39  93/05/02  18:00:50  18:00:50  jraja (Jarno Tapio Rajahalme)
 * Added include for sys/socket.h (net/route.h needs it.).
 * 
 * Revision 1.38  93/05/02  17:28:01  17:28:01  jraja (Jarno Tapio Rajahalme)
 * Changed include route_protos.h to include route.h.
 * 
 * Revision 1.37  93/04/28  12:57:11  12:57:11  puhuri (Markus Peuhkuri)
 * Fixed stdargs.h to stdarg.h
 * 
 * Revision 1.36  93/04/27  10:20:16  10:20:16  puhuri (Markus Peuhkuri)
 * Add Syslog.
 * 
 * Revision 1.35  93/04/26  11:52:28  11:52:28  too (Tomi Ollila)
 * Changed include paths of amiga_api.h, amiga_libcallentry.h and amiga_raf.h
 * from kern to api
 * 
 * Revision 1.34  93/04/24  23:43:24  23:43:24  jraja (Jarno Tapio Rajahalme)
 * Removed few #ifdef AMITCP's.
 * 
 * Revision 1.33  93/04/13  22:24:28  22:24:28  jraja (Jarno Tapio Rajahalme)
 * Changed make_id to take LONG instead of LONG * and to return the id
 * as the return value (-1 indicates error).
 * 
 * Revision 1.32  93/04/12  02:49:41  02:49:41  jraja (Jarno Tapio Rajahalme)
 * Changed Release*Socket to use getSock().
 * Changed ReleaseSocket() to set owner of the socket to NULL.
 * 
 * Revision 1.31  93/04/12  00:39:14  00:39:14  jraja (Jarno Tapio Rajahalme)
 * Fixed type (task -> Task).
 * 
 * Revision 1.30  93/04/12  00:20:24  00:20:24  jraja (Jarno Tapio Rajahalme)
 * Changed FIO[SG]ETOWN and SIOC[SG]PGRP to take struct Task * instead of pid.
 * 
 * Revision 1.29  93/04/12  00:00:06  00:00:06  jraja (Jarno Tapio Rajahalme)
 * Enabled sigmp with tsleep_main.
 * Commented out syscall_semaphores.
 * 
 * Revision 1.28  93/04/11  22:27:31  22:27:31  jraja (Jarno Tapio Rajahalme)
 * Added ()'s around << with -.
 * 
 * Revision 1.27  93/04/07  11:36:43  11:36:43  too (Tomi Ollila)
 * domain, type and protocol active in ObtainSocket. Small change made
 * to get unique id correctly
 * 
 * Revision 1.26  93/04/06  17:59:19  17:59:19  too (Tomi Ollila)
 * Added thandling of EINTR to WaitSelect (former Select). does not 
 * use sigmp yet. added SetSOcketSignals()
 * 
 * Revision 1.25  93/04/06  15:15:13  15:15:13  jraja (Jarno Tapio Rajahalme)
 * Changed spl function return value storage to spl_t,
 * changed bcopys and bzeros to aligned and/or const when possible,
 * added inclusion of conf.h to every .c file.
 * 
 * Revision 1.24  93/03/30  16:56:59  16:56:59  too (Tomi Ollila)
 * Made Obtain/Release[CopyOf]Socket active
 * 
 * Revision 1.23  93/03/29  19:29:03  19:29:03  too (Tomi Ollila)
 * Added releaseSocket(), releaseCopyOfSOcket() and ObtainSOcket().
 * Not functional yet.
 * 
 * Revision 1.22  93/03/19  14:14:37  14:14:37  too (Tomi Ollila)
 * Code changes at night 17-18 March 1993
 * 
 * Revision 1.21  93/03/16  08:30:30  08:30:30  jraja (Jarno Tapio Rajahalme)
 * Removed redundant include of sys/stat.h.
 * 
 * Revision 1.20  93/03/12  23:58:24  23:58:24  ppessi (Pekka Pessi)
 * Added inte_addr() return tyep.
 * 
 * Revision 1.19  93/03/12  16:54:26  16:54:26  too (Tomi Ollila)
 * Added SetErrno()
 * 
 * Revision 1.18  93/03/12  15:20:02  15:20:02  too (Tomi Ollila)
 * Added getdtablesize()
 * Added handling of field nextDToSearch in socketbase, if 
 * it's value is bigged than descriptor of closed socket.
 * 
 * Revision 1.17  93/03/10  22:12:20  22:12:20  too (Tomi Ollila)
 * 
 * Changed Errno() to use libPtr->errnoPtr, select now has
 * timeout argument in register A3
 * 
 * Revision 1.16  93/03/09  17:52:04  17:52:04  ppessi (Pekka Pessi)
 *  Fixed bug with RW ioctls
 * 
 * Revision 1.15  93/03/07  17:27:36  17:27:36  too (Tomi Ollila)
 * Added SetDTableSize() and fized refcnt to so_refcnt.
 * Timeout argument of Select NOT changed back to go to a3, so
 * gcc 2.2.2 can compile it.
 * 
 * Revision 1.14  93/03/07  11:56:51  11:56:51  too (Tomi Ollila)
 * Added reference count usage to CloseSocket
 * Changed Ioctl() RETURN()s to be consistent w/ other API code.
 * ,
 * 
 * Revision 1.13  93/03/07  00:52:09  00:52:09  jraja (Jarno Tapio Rajahalme)
 * Added keyword SAVEDS to api function definitions.
 * 
 * Revision 1.12  93/03/06  02:11:38  02:11:38  jraja (Jarno Tapio Rajahalme)
 * Added API-functions Errno() and CloseSocket().
 * 
 * Revision 1.11  93/03/05  21:11:09  21:11:09  jraja (Jarno Tapio Rajahalme)
 * Fixed includes (again).
 * 
 * Revision 1.10  93/03/05  03:26:04  03:26:04  ppessi (Pekka Pessi)
 * Compiles with SASC. Initial test version.
 * 
 * Revision 1.9  93/03/02  14:31:24  14:31:24  ppessi (Pekka Pessi)
 * 	Fixed selenter argument 'hdr' type
 * 
 * Revision 1.8  93/02/27  11:03:37  11:03:37  too (Tomi Ollila)
 * Added prototypes
 * 
 * Revision 1.7  93/02/26  13:22:12  13:22:12  too (Tomi Ollila)
 * code checked w/ too, ppessi and jraja
 * 
 * Revision 1.6  93/02/25  19:13:55  19:13:55  too (Tomi Ollila)
 * Added ffs() inline function
 * 
 * Revision 1.5  93/02/25  17:52:52  17:52:52  too (Tomi Ollila)
 * Added itimerfix as inline function.
 * 
 * Revision 1.4  93/02/25  16:47:41  16:47:41  too (Tomi Ollila)
 * Compiles (argument timeout is now put into data register (d0)
 * 
 * Revision 1.3  93/02/24  19:23:01  19:23:01  too (Tomi Ollila)
 * *** empty log message ***
 * 
 * Revision 1.2  93/02/24  18:29:41  18:29:41  too (Tomi Ollila)
 * cosmetic changes ?.
 * 
 * Revision 1.1  93/02/23  17:54:30  17:54:30  too (Tomi Ollila)
 * Initial revision
 * 
 */

#include <conf.h>

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/socketvar.h>
#include <sys/kernel.h>
#include <sys/ioctl.h>
#include <sys/protosw.h>
#include <sys/malloc.h>
#include <sys/synch.h>

#include <sys/time.h>
#include <sys/errno.h>

#include <sys/socket.h>
#include <net/route.h>

#include <kern/amiga_includes.h>

#include <api/amiga_api.h>
#include <api/amiga_libcallentry.h>
#include <api/allocdatabuffer.h>

#include <api/apicalls.h>

#include <stdarg.h>

#include <net/if_protos.h>

#include <kern/uipc_domain_protos.h>
#include <kern/uipc_socket_protos.h>
#include <kern/uipc_socket2_protos.h>

/* Local protos */
int selscan(struct SocketBase *p, 
	    fd_mask *in, fd_mask *ou, fd_mask *ex, fd_mask * obits,
	    int nfd, int * retval, int * selitemcount_p);
void selenter(struct SocketBase * p, struct newselitem ** hdr);
void unselect(register struct newselbuf * sb);
void selwakeup(struct newselitem **hdr);
int soo_select(struct socket *so, int which, struct SocketBase *p);
int countSockets(struct SocketBase * libPtr, struct socket * so);

/*
 * itimerfix copied from bsdss/server/kern/kern_time.c. since fields
 * in struct timeval in amiga are ULONGs, values less than zero need
 * not be checked. the second check, timeval less than resolution of
 * the clock is not needed in amiga...hmm, is removed also.
 */
static inline int itimerfix(struct timeval *tv)
{
	if (tv->tv_sec > 100000000 || tv->tv_usec >= 1000000)
		return (EINVAL);
/*	if (tv->tv_sec == 0 && tv->tv_usec != 0 && tv->tv_usec < tick)
		tv->tv_usec = tick;
*/	return (0);
}

/*
 * ffs() copied directly from bsdss/server/kern/subr_xxx.c
 */
static inline int ffs(register long mask)
{
  register int bit;

  if (!mask)
    return(0);
  for (bit = 1;; ++bit) {
    if (mask&0x01)
      return(bit);
    mask >>= 1;
  }
  /* NOT REACHED */
}


LONG /* SAVEDS */ RAF1(_Errno,
		       struct SocketBase *, libPtr, a6)
#if 0
{
#endif
  return (LONG)libPtr->errnoPtr[libPtr->errnoSize - 1];
}

VOID /* SAVEDS */ RAF3(_SetErrnoPtr,
		       struct SocketBase *,	libPtr,	a6,
		       VOID *,			err_p,	a0,
		       UBYTE,			size,	d0)
#if 0     
{
#endif

  libPtr->errnoSize = size;
  libPtr->errnoPtr = err_p;
}


/*
 * Ioctl system call
 */

LONG SAVEDS RAF4(_IoctlSocket,
		 struct SocketBase *,	libPtr,	a6,
		 LONG,			fdes,	d0,
		 ULONG,		cmd,	d1,
		 caddr_t,		data,	a0)
#if 0
{
#endif

  register int error;
  register u_int size;
  struct socket *so;

  /*
   * Note: Syscall semaphore is essential here if two processes can access
   * the same socket. (can it be changed to spl_? ?)
   */
  CHECK_TASK();
  ObtainSyscallSemaphore(libPtr);
  
  if (error = getSock(libPtr, fdes, &so))
    goto Return;

  /*
   * Interpret high order word to get size of the user data area
   */
  size = IOCPARM_LEN(cmd);
  if (size == 0 || size > IOCPARM_MAX) {
    error = ENOTTY;
    goto Return;
  }
  if (!(cmd & IOC_IN) && cmd & IOC_OUT)
    /*
     * Zero the buffer so the user always
     * gets back something deterministic.
     */
    bzero(data, size);
  
  switch (cmd) {
  case FIOCLEX:
  case FIONCLEX:		/* should this return error ???? */
    goto Return;

  case FIOSETOWN:
  case SIOCSPGRP:
    /*
     * data is a struct Task **, find corresponding SocketBase * and set owner
     */
    so->so_pgid = FindSocketBase(*(struct Task **)data);
    goto Return;
    
  case FIOGETOWN:
  case SIOCGPGRP:
    if (so->so_pgid)
      *(struct Task **)data = so->so_pgid->thisTask;
    else
      *(struct Task **)data = NULL;
    goto Return;

  case FIONBIO:
    if (*(int *)data)
      so->so_state |= SS_NBIO;
    else
      so->so_state &= ~SS_NBIO;
    goto Return;
    
  case FIOASYNC:
    if (*(int *)data) {
      so->so_state |= SS_ASYNC;
      so->so_rcv.sb_flags |= SB_ASYNC;
      so->so_snd.sb_flags |= SB_ASYNC;
    } else {
      so->so_state &= ~SS_ASYNC;
      so->so_rcv.sb_flags &= ~SB_ASYNC;
      so->so_snd.sb_flags &= ~SB_ASYNC;
    }
    goto Return;
    
  case FIONREAD:
    *(int *)data = so->so_rcv.sb_cc;
    goto Return;
    
  case SIOCATMARK:
    *(int *)data = (so->so_state&SS_RCVATMARK) != 0;
    goto Return;
  }

  /*
   * Interface/routing/protocol specific ioctls:
   * interface and routing ioctls should have a
   * different entry since a socket's unnecessary -- not really,
   * ifioctl needs the socket (?)
   */
  if (IOCGROUP(cmd) == 'I') {
    error = (ifioctl(so, cmd, data));
    goto Return;
  }
  if (IOCGROUP(cmd) == 'r') {
    error = (rtioctl(cmd, data));
    goto Return;
  }
  error = ((*so->so_proto->pr_usrreq)(so, PRU_CONTROL, 
	  (struct mbuf *)cmd, (struct mbuf *)data, (struct mbuf *)0));

 Return:
  ReleaseSyscallSemaphore(libPtr);
  API_STD_RETURN(error, 0);
}


/*
 *  Semaphore to prevent select buffers from simultaneous modifications
 */
struct SignalSemaphore select_semaphore = { 0 };

struct newselbuf {
  int s_state;
#define	SB_CLEAR	0		/* wait condition cleared */
#define	SB_WILLWAIT	1		/* will wait if not cleared */
#define	SB_WAITING	2		/* waiting - wake up */
  int s_count;
  struct newselitem {
    struct	newselitem  *	si_next;	/* next selbuf in item chain */
    struct	newselitem **	si_prev;	/* back pointer */
    struct	newselbuf   *	si_selbuf;	/* 'thread' waiting */
  } s_item[0]; /* selitems are allocated at select when right
		  number of descriptors are known */
};

void select_init(void)
{
  InitSemaphore(&select_semaphore);
}

/*
 *  symbolic names FREAD and FWRITE aren'n needed anywhere else if only
 *  socket descriptors are in use.
 */
#define FREAD	0x1
#define FWRITE	0x2

/*
 * Select(), selscan(), selenter(), selwakeup() and unselect() are taken from
 * ../server/kern/sys_generic.h in bsdss distribution. soo_select() is
 * taken from ../server/kern/sys_socket.h, same distribution.
 */

LONG SAVEDS RAF7(_WaitSelect,
		 struct SocketBase *,	libPtr,		a6,
		 ULONG,			nfds,		d0,
		 fd_mask *,		readfds,	a0,
		 fd_mask *,		writefds,	a1,
		 fd_mask *,		exeptfds,	a2,
		 struct timeval *,	timeout,	a3,
		 ULONG *,		sigmp,		d1)
#if 0
{
#endif
  
  fd_mask * obits;
  u_int obitsize;  /* in bytes */
  int error, retval;
  int selitemcount;
  ULONG sigmask = sigmp ? *sigmp : 0;

  struct newselbuf * newselbuf;
  
  CHECK_TASK();

  if (nfds > libPtr->dTableSize)
    nfds = libPtr->dTableSize;	/* forgiving; slightly wrong */
  
  if (timeout && itimerfix(timeout)) {
    error = EINVAL;
    goto Return;
  }
  
  /*
   *	We use selscan twice each time around the loop.
   *	The first time, we actually put ourself on device chains.
   *	The second time, we make a "fast" check for a true condition.
   *	selenter does nothing and we don't need to use unselect.
   *	We only need to zero obits once because if selscan
   *	turns on a bit it returns non-zero and we stop.
   *
   *	The "fast" scan is also done before the loop is ever entered.
   *
   *	We need to allocate some temporary buffers for output bit masks
   *	and, later for selitems. The amount of selitems needed is
   *	calculated during the first selscan -- when selenter does nothing
   */

  obitsize = howmany(nfds, NFDBITS) * sizeof (fd_mask);
  if (allocDataBuffer(&libPtr->selitems, 3 * obitsize) == FALSE) {
    error = ENOMEM;
    goto Return;
  }
  obits = (fd_mask *)libPtr->selitems.db_Addr;

  aligned_bzero((caddr_t)obits, 3 * obitsize);

  /*
   *	We make a first pass to check for conditions
   *	which are already true.  Clearing uu_sb
   *	will keep selenter from doing anything.
   */
  libPtr->p_sb = 0;
  error = selscan(libPtr, readfds, writefds, exeptfds,
		  obits, nfds, &retval, &selitemcount);

  /*
   *	Check if the 'real' loop should be entered.
   */
  if (!error && !retval) {
    if (timeout == NULL || timeout->tv_secs != 0L || timeout->tv_micro != 0L) {
	
      /*
       * Send timeout if there is any.
       */
      if (timeout)
	tsleep_send_timeout(libPtr, timeout);

      /*
       * Now, make sure there is room for all the selitems...
       */
      if (allocDataBuffer(&libPtr->selitems,
			  3 * obitsize + sizeof (struct newselbuf) +
			  selitemcount * sizeof(struct newselitem)) == FALSE) {
	error = ENOMEM;
	goto Return;
      }
      /*
       * We need to clear obits again _if_ the memory area for it has 
       * changed.
       */
      if ((fd_mask *)libPtr->selitems.db_Addr != obits) {
	obits = (fd_mask *)libPtr->selitems.db_Addr;
	aligned_bzero((caddr_t)obits, 3 * obitsize);
      }
      newselbuf = (struct newselbuf *)((caddr_t)obits + (3 * obitsize));
      
      while (TRUE) {

	/*
	 *	Now we get serious about blocking.
	 *	selenter will put us on device chains.
	 *	We don't need select_lock here because
	 *	no other thread can get at selbuf yet.
	 */
	
	newselbuf->s_state = SB_WILLWAIT;
	newselbuf->s_count = 0;
	libPtr->p_sb = newselbuf;
	
	error = selscan(libPtr, readfds, writefds, exeptfds,
			obits, nfds, &retval, &selitemcount);
	if (error || retval) {
	  ObtainSemaphore(&select_semaphore);
	  unselect(newselbuf);
	  ReleaseSemaphore(&select_semaphore);
	  break;
	}
	
	/*
	 *	If the state is already SB_CLEAR, then a selwakeup
	 *	slipped in after the selscan.
	 */
	
	ObtainSemaphore(&select_semaphore);
	if (newselbuf->s_state == SB_CLEAR) {
	  unselect(newselbuf);
	  ReleaseSemaphore(&select_semaphore);
	}
	else {
	  newselbuf->s_state = SB_WAITING;
	  tsleep_enter(libPtr, (caddr_t)newselbuf, "select");
	  ReleaseSemaphore(&select_semaphore);
	  /* ReleaseSemaphore(&syscall_semaphore); don't have this */
	  
	  error = tsleep_main(libPtr, sigmask);
	  
	  /* ObtainSemaphore(&syscall_semaphore); see above */
	  ObtainSemaphore(&select_semaphore);
	  unselect(newselbuf);
	  ReleaseSemaphore(&select_semaphore);
	  
	  if (error != 0) {
	    /*
	     * The break at the end of this block means that selscan will NOT 
	     * be done after this. This provides faster response to the 
	     * used defined signals.
	     *
	     * Note also that the semantics of the tsleep_abort_timeout() has 
	     * changed; it is now accepted to call it even after the timeout
	     * has expired.
	     */
	    if (error == ERESTART || error == EWOULDBLOCK)
	      error = 0;
	    
	    break;		/* do not scan now */
	  }
	  
	}
	/*
	 *	Scan for true conditions. Clearing uu_sb
	 *	will keep selenter from doing anything.
	 */
	libPtr->p_sb = 0;
	error = selscan(libPtr, readfds, writefds, exeptfds,
			obits, nfds, &retval, &selitemcount);
	if (error || retval)
	  break;
      } /* while (TRUE) */	

      if (timeout)		/* abort the timeout if any */
	tsleep_abort_timeout(libPtr, timeout);
    }
  }

 Return:
  libPtr->p_sb = 0;

#define	putbits(name, x) \
  if (name) { \
    aligned_bcopy(((caddr_t)obits) + (x * obitsize), \
		  (caddr_t)name, obitsize); \
   }

  if (error == 0) {
    if (sigmp)
      *sigmp &= SetSignal(0L, sigmask);
    
    putbits(readfds, 0);
    putbits(writefds, 1);
    putbits(exeptfds, 2);
  }
#undef putbits

  API_STD_RETURN(error, retval);
}

int selscan(struct SocketBase *p,
	    fd_mask * in,
	    fd_mask * ou,
	    fd_mask * ex,
	    fd_mask * obits,
	    int nfd,
	    int * retval,
	    int * selitemcount_p)

{
	register int which, i, j, selitemcount = 0;
	register fd_mask bits;
	register fd_mask * cbits;
	struct socket *so;
	int flag;
	int n = 0;
	int error = 0;
	u_int obitsize = howmany(nfd, NFDBITS); /* in longwords */

	for (which = 0; which < 3; which++) {
		switch (which) {

		case 0:
			flag = FREAD;
			cbits = in;
			break;
		case 1:
			flag = FWRITE;
			cbits = ou;
			break;
		case 2:
			flag = 0;
			cbits = ex;
			break;
		}
		if (cbits != NULL) {
		  for (i = 0; i < nfd; i += NFDBITS) {
			bits = cbits[i/NFDBITS];
			while ((j = ffs(bits)) && i + --j < nfd) {
				bits &= ~(1 << j);
				so = p->dTable[i + j];
				if (so == NULL) {
				  error = EBADF;
				  break;
				}
				selitemcount++;
				if (soo_select(so, flag, p)) {
				  FD_SET(i + j, (fd_set *)
					 (obits + (which * obitsize)));
				  n++;
				}
			}
		  }
		}
	}
	*retval = n;
	*selitemcount_p = selitemcount;
	return (error);
}

/*
 * Add a select event to the current select buffer.
 * Chain all select buffers that are waiting for
 * the same event.
 */
void selenter(struct SocketBase *	p,
	      struct newselitem **	hdr)
{
	register struct newselbuf *sb = p->p_sb;

	if (sb) {
	    register struct newselitem **sip = hdr;
	    register struct newselitem *si;

	    ObtainSemaphore(&select_semaphore);

	    /* initialize this select item */
	    si = &sb->s_item[sb->s_count++];
	    si->si_selbuf = sb;

	    /* and add it to the device chain */
	    if (si->si_next = *sip)
		si->si_next->si_prev = &si->si_next;
	    *(si->si_prev = sip) = si;

	    ReleaseSemaphore(&select_semaphore);
	}
}

void unselect(register struct newselbuf * sb)
{
	int i;

	/*
	 *	We unchain the select items.
	 *	This assumes select_lock is held.
	 */

	for (i = 0; i < sb->s_count; i++) {
	    register struct newselitem *si = &sb->s_item[i];

	    /*
	     *	We remove this item from its device chain.
	     *	If selwakeup already got to it, then this is a nop.
	     */

	    *si->si_prev = si->si_next;
	}
}

/*
 * Wakeup all threads waiting on the same select event.
 * Do not clear the other select events that each thread
 * is waiting on; thread will do that itself.
 */
void selwakeup(struct newselitem **hdr)
{
	register struct newselitem **sip;
	register struct newselitem *si;

	ObtainSemaphore(&select_semaphore);

	for (sip = hdr; (si = *sip) != 0; sip = &si->si_next) {
	    register struct newselbuf *sb;
	    int old_state;

	    /* for when unselect tries to dequeue this item */
	    si->si_prev = &si->si_next;

	    /* check for wakeup */
	    sb = si->si_selbuf;
	    old_state = sb->s_state;
	    sb->s_state = SB_CLEAR;
	    if (old_state == SB_WAITING)
		wakeup((caddr_t)sb);
	}

	/* clear the device chain */
	*(struct newselitem **)hdr = 0;
	ReleaseSemaphore(&select_semaphore);
}

int soo_select(struct socket *so,
	       int which,
	       struct SocketBase *p)
{
	register spl_t s = splnet();

	switch (which) {

	case FREAD:
		if (soreadable(so)) {
			splx(s);
			return (1);
		}
		sbselqueue(&so->so_rcv, p);
		break;

	case FWRITE:
		if (sowriteable(so)) {
			splx(s);
			return (1);
		}
		sbselqueue(&so->so_snd, p);
		break;

	case 0:
		if (so->so_oobmark ||
		    (so->so_state & SS_RCVATMARK)) {
			splx(s);
			return (1);
		}
		sbselqueue(&so->so_rcv, p);
		break;
	}
	splx(s);
	return (0);
}

/*
 * countSockets() counts how many references ONE task (SocketBase) have to a
 * socket
 */

int countSockets(struct SocketBase * libPtr, struct socket * so)
{
  int i, count = 0;

  for (i = 0; i < libPtr->dTableSize; i++)
    if (libPtr->dTable[i] == so)
      count++;

  return count;
}

LONG SAVEDS RAF2(_CloseSocket,
		 struct SocketBase *,	libPtr,	a6,
		 LONG,			d,	d0)
#if 0
{
#endif

  register int error;
  struct socket *so;

  CHECK_TASK();
  ObtainSyscallSemaphore(libPtr);
  
  if (error = getSock(libPtr, d, &so))
    goto Return;

  if (so->so_pgid == libPtr && countSockets(libPtr, so) == 1) 
    so->so_pgid = NULL;		/* not ours any more */

  /*
   * Decrease the reference count of a socket (AmiTCP addition) and return if
   * not zero.
   */
  if (--so->so_refcnt <= 0)
    error = soclose(so);

  /*
   * clear the file descriptor entry and set nextDToSearch to point
   * its place is it had bigger value.
   */
  libPtr->dTable[d] = NULL;
  if (d < libPtr->nextDToSearch)
    libPtr->nextDToSearch = d;

 Return: 
  ReleaseSyscallSemaphore(libPtr);
  API_STD_RETURN(error, 0);
}

/*
 * Set size of descriptor tab|e
 */

LONG SAVEDS RAF2(_SetDTableSize,
		 struct SocketBase *,	libPtr, a6,
		 UWORD,			size,	d0)
#if 0
{
#endif
  
  register LONG	oldsize = (LONG)libPtr->dTableSize;
  register LONG copysize;
  register struct socket ** dTable;
  
  CHECK_TASK();

  if (size < oldsize) {
    int i;

    for (i = oldsize - 1; i >= size; i--)
      if (libPtr->dTable[i] != NULL)
	break;
    size = i + 1;
    copysize = size;
  }
  else
    copysize = oldsize;

  if ((dTable = AllocMem(size * sizeof (struct socket *),
			 MEMF_PUBLIC|MEMF_CLEAR)) == NULL)
    return oldsize;

  aligned_bcopy(libPtr->dTable, dTable, copysize * sizeof (struct socket *));

  FreeMem(libPtr->dTable, oldsize * sizeof (struct socket *));
  
  libPtr->dTable = dTable;
  libPtr->dTableSize = size;
  
  return size;
}

LONG /* SAVEDS */ RAF1(_getdtablesize,
		       struct SocketBase *,	libPtr, a6)
#if 0
{
#endif

  return libPtr->dTableSize;
}

VOID SAVEDS RAF4(_SetSocketSignals,
		 struct SocketBase *,	libPtr, 	a6,
		 ULONG,			SIGINTR,	d0,
		 ULONG,			SIGIO,		d1,
		 ULONG,			SIGURG,		d2)
#if 0
{
#endif
  CHECK_TASK(); /* you may get compiler warnings here, but ignore those */
  /*
   * The operations below are atomic so no need to protect them
   */
  libPtr->sigIntrMask = SIGINTR;
  libPtr->sigIOMask = SIGIO;
  libPtr->sigUrgMask = SIGURG;
}

struct SocketNode {
  struct MinNode	sn_Node;
  LONG			sn_Id;
  struct socket *	sn_Socket;
};

/*
 * checkId() checks that there are no released sockets w/ given id.
 * used from function makeId().
 */

static LONG checkId(int id)
{
  struct Node * sn;

  Forbid();
  for (sn = releasedSocketList.lh_Head; sn->ln_Succ; sn = sn->ln_Succ)
    if (((struct SocketNode *)sn)->sn_Id == id) {
      Permit();
      return EINVAL;
    }
  Permit();	
  return 0;
}

#define FIRSTUNIQUEID 65536

/*
 * makeId() gets next unique id for socket to be released if *id is -1
 * otherwise calls checkId for given *id.
 * used from functions ReleaseSocket() and ReleaseCopyOfSocket().
 */
static LONG makeId(LONG id)
{
  static int uniqueId = FIRSTUNIQUEID;

  if (id == UNIQUE_ID) {
    do
      if (++uniqueId == (1<<31) - 1)
	uniqueId = FIRSTUNIQUEID;
    while (checkId(uniqueId) != 0);

    return uniqueId;
  }
  else {
    if (checkId(id) == 0)
      return id;
    else
      return -1; /* error */
  }
}

LONG SAVEDS RAF3(_ReleaseSocket,
		 struct SocketBase *,	libPtr, a6,
		 LONG,			fd,	d0,
		 LONG,			id,	d1)
#if 0		 
{
#endif
  struct SocketNode *sn;
  struct socket *so;
  int error = 0;

  CHECK_TASK();

  if ((ULONG)id >= FIRSTUNIQUEID && (id = makeId(id)) == -1) {
    error = EINVAL;
    goto Return;
  }

  if ((error = getSock(libPtr, fd, &so)) != 0)
    goto Return;
 
  if ((sn = AllocMem(sizeof (struct SocketNode), MEMF_PUBLIC)) == NULL) {
    error = ENOMEM;
    goto Return;
  }
  if (so->so_pgid == libPtr && countSockets(libPtr, so) == 1) 
    so->so_pgid = NULL;		/* not ours any more */
  sn->sn_Id = id;
  sn->sn_Socket = so;
  libPtr->dTable[fd] = NULL;
  if (fd < libPtr->nextDToSearch)
    libPtr->nextDToSearch = fd;

  Forbid();
  AddTail(&releasedSocketList, (struct Node *)sn);
  Permit();

 Return: API_STD_RETURN(error, id);
}

LONG SAVEDS RAF3(_ReleaseCopyOfSocket,
		 struct SocketBase *,	libPtr, a6,
		 LONG,			fd,	d0,
		 LONG,			id,	d1)
#if 0		 
{
#endif
  struct SocketNode *sn;
  struct socket *so;
  int error = 0;

  CHECK_TASK();

  if ((ULONG)id >= FIRSTUNIQUEID && (id = makeId(id)) == -1) {
    error = EINVAL;
    goto Return;
  }

  if ((error = getSock(libPtr, fd, &so)) != 0)
    goto Return;

  if ((sn = AllocMem(sizeof (struct SocketNode), MEMF_PUBLIC)) == NULL) {
    error = ENOMEM;
    goto Return;
  }
  /* so_pgid left intact */
  sn->sn_Id = id;
  sn->sn_Socket = so;
  so->so_refcnt++;
  Forbid();
  AddTail(&releasedSocketList, (struct Node *)sn);
  Permit();

 Return: API_STD_RETURN(error, id);
}

LONG SAVEDS RAF5(_ObtainSocket,
		 struct SocketBase *,	libPtr, 	a6,
		 LONG,			id,		d0,
		 LONG,			domain,		d1,
		 LONG,			type,		d2,
		 LONG,			protocol,	d3)
#if 0
{
#endif

  struct protosw *prp;
  struct Node * sn;
  int error = 0;
  LONG fd;

  CHECK_TASK();

  if (protocol)
    prp = pffindproto(domain, protocol, type);
  else
    prp = pffindtype(domain, type);
  if (prp == 0)
    error = EPROTONOSUPPORT;
  if (prp->pr_type != type)
    error = EPROTOTYPE;
  if (error)
    goto Return;
  
  if ((error = fdAlloc(libPtr, &fd)) != 0)
    goto Return;

  Forbid();
  for (sn = releasedSocketList.lh_Head; sn->ln_Succ; sn = sn->ln_Succ)
    if (((struct SocketNode *)sn)->sn_Id == id) {
      if (prp != ((struct SocketNode *)sn)->sn_Socket->so_proto)
	continue;
      Remove(sn);
      Permit();
      libPtr->dTable[fd] = ((struct SocketNode *)sn)->sn_Socket;
      FreeMem(sn, sizeof (struct SocketNode));
      goto Return;
    }
  /* here if fdAlloc succeeded but search of socket failed */
  Permit();
  error = EWOULDBLOCK;
  libPtr->nextDToSearch = fd;
  
 Return: API_STD_RETURN(error, fd);
}

VOID SAVEDS RAF4(_Syslog,
		 struct SocketBase *,	libPtr, 	a6,
		 ULONG,			level,		d0,
		 const STRPTR,		format,         a0,
		 va_list,		ap,		a1)
#if 0
{
#endif
  vlog(level, format, ap);
}

LONG SAVEDS RAF3(_Dup2Socket,
		 struct SocketBase *,	libPtr, 	a6,
		 LONG,			fd1,		d0,
		 LONG,			fd2,		d1)
#if 0     
{
#endif
  LONG newfd;
  int error;
  struct socket *so;

  CHECK_TASK();
  ObtainSyscallSemaphore(libPtr); /*many tasks may have access to the socket */

  if (error = getSock(libPtr, fd1, &so))
    goto Return;
    
  if (fd2 == -1)
    if (error = fdAlloc(libPtr, &newfd))
      goto Return;
    else
      fd2 = newfd;
  else {
    if ((ULONG)fd2 >= libPtr->dTableSize) {
      error = EBADF;
      goto Return;
    }
    if (libPtr->dTable[fd2] != NULL)
      CloseSocket(libPtr, fd2);
  }

  so->so_refcnt++;
  libPtr->dTable[fd2] = so;

 Return:
  ReleaseSyscallSemaphore(libPtr);
  API_STD_RETURN(error, fd2);
}

