RCS_ID_C="$Id: amiga_libtables.c,v 1.20 1993/08/12 07:32:27 jraja Exp $";
/*
 * Copyright (c) 1993 AmiTCP/IP Group, <amitcp-group@hut.fi>
 *                    Helsinki University of Technology, Finland.
 *                    All rights reserved.
 * 
 * Created: Tue Feb 16 14:14:33 1993 too
 * Last modified: Tue Jun 29 08:49:46 1993 too
 * 
 * HISTORY 
 * $Log: amiga_libtables.c,v $
 * Revision 1.20  1993/08/12  07:32:27  jraja
 * Changed ioctl to IoctlSocket (too).
 *
 * Revision 1.19  1993/06/12  08:57:05  too
 * Added Du2Socket()
 *
 * Revision 1.18  1993/06/04  11:16:15  jraja
 * Fixes for first public release.
 *
 * Revision 1.17  1993/05/17  01:07:47  ppessi
 * Changed RCS version.
 *
 * Revision 1.16  1993/05/04  14:08:41  too
 * Changed local api function names to the same as the names given
 * to API programmer, but w/ starting _
 *
 * Revision 1.15  93/04/29  02:21:30  02:21:30  jraja (Jarno Tapio Rajahalme)
 * Added function rresvport().
 * 
 * Revision 1.14  93/04/27  10:19:50  10:19:50  puhuri (Markus Peuhkuri)
 * Add entry for Syslog.
 * 
 * Revision 1.13  93/04/26  12:49:00  12:49:00  too (Tomi Ollila)
 * Added getXbY functions
 * 
 * Revision 1.12  93/04/26  11:52:37  11:52:37  too (Tomi Ollila)
 * Changed include paths of amiga_api.h, amiga_libcallentry.h and amiga_raf.h
 * from kern to api
 * 
 * Revision 1.11  93/04/13  22:29:01  22:29:01  jraja (Jarno Tapio Rajahalme)
 * Added New functions and Null's for the database functions.
 * 
 * Revision 1.10  93/04/06  15:15:23  15:15:23  jraja (Jarno Tapio Rajahalme)
 * Changed spl function return value storage to spl_t,
 * changed bcopys and bzeros to aligned and/or const when possible,
 * added inclusion of conf.h to every .c file.
 * 
 * Revision 1.9  93/03/30  16:58:22  16:58:22  too (Tomi Ollila)
 * Added some functions and space for those not yet implemented.
 * 
 * Revision 1.8  93/03/13  17:12:31  17:12:31  ppessi (Pekka Pessi)
 * Fixed bugs with variable initializations. Works with UDP.
 * 
 * Revision 1.7  93/03/12  16:53:16  16:53:16  too (Tomi Ollila)
 * Added SetErrno() just after Errno()
 * 
 * Revision 1.6  93/03/07  00:48:56  00:48:56  jraja (Jarno Tapio Rajahalme)
 * Fixed function declarations by adding REGARGFUN keywords,
 * since functions with register arguments have different link time names
 * than normal ones.
 * 
 * Revision 1.5  93/03/06  02:10:58  02:10:58  jraja (Jarno Tapio Rajahalme)
 * Added CloseSocket, Errno and inet_* functions.
 * 
 * Revision 1.4  93/03/05  15:52:29  15:52:29  too (Tomi Ollila)
 * Added gethostbyname() and GetPeerName()
 * 
 * Revision 1.3  93/03/03  23:54:41  23:54:41  too (Tomi Ollila)
 * Added 14 api functions.
 * 
 * Revision 1.2  93/02/25  16:44:01  16:44:01  too (Tomi Ollila)
 * merged ExecLibraryList_functable and its "declarations" here.
 * 
 * Revision 1.1  93/02/24  19:01:44  19:01:44  too (Tomi Ollila)
 * Initial revision
 * 
 *
 */

#include <conf.h>

#include <exec/types.h>
#include <sys/param.h>
#include <api/amiga_raf.h>

typedef VOID (* REGARGFUN f_void)();

/*
 * Null used in both function tables
 */
extern VOID Null(VOID);

/*
 * "declarations" for ExecLibraryList_funcTable functions.
 */ 

extern REGARGFUN VOID ELL_Open();
extern REGARGFUN VOID ELL_Expunge();

f_void ExecLibraryList_funcTable[] = {
  ELL_Open,
  Null,		/* ELL_Close() is never called */
  ELL_Expunge,
  Null,		/* ELL_Reserved() */
  (f_void)-1
};

/*
 * "declarations" for userLibrary_funcTable functions.
 */ 
extern REGARGFUN VOID UL_Close();

extern REGARGFUN VOID _socket();
extern REGARGFUN VOID _bind();
extern REGARGFUN VOID _listen();
extern REGARGFUN VOID _accept();
extern REGARGFUN VOID _connect();
extern REGARGFUN VOID _sendto();
extern REGARGFUN VOID _send();
extern REGARGFUN VOID _recvfrom();
extern REGARGFUN VOID _recv();
extern REGARGFUN VOID _shutdown();
extern REGARGFUN VOID _setsockopt();
extern REGARGFUN VOID _getsockopt();
extern REGARGFUN VOID _getsockname();
extern REGARGFUN VOID _getpeername();

extern REGARGFUN VOID _IoctlSocket();
extern REGARGFUN VOID _CloseSocket();
extern REGARGFUN VOID _WaitSelect();
extern REGARGFUN VOID _SetSocketSignals();
extern REGARGFUN VOID _SetDTableSize();
extern REGARGFUN VOID _ObtainSocket();
extern REGARGFUN VOID _ReleaseSocket();
extern REGARGFUN VOID _ReleaseCopyOfSocket();
extern REGARGFUN VOID _Errno();
extern REGARGFUN VOID _SetErrnoPtr();

extern REGARGFUN VOID _Inet_NtoA();
extern REGARGFUN VOID _inet_addr();
extern REGARGFUN VOID _Inet_LnaOf();
extern REGARGFUN VOID _Inet_NetOf();
extern REGARGFUN VOID _Inet_MakeAddr();
extern REGARGFUN VOID _inet_network();

extern REGARGFUN VOID _gethostbyname();
extern REGARGFUN VOID _gethostbyaddr();
extern REGARGFUN VOID _getnetbyname();
extern REGARGFUN VOID _getnetbyaddr();
extern REGARGFUN VOID _getservbyname();
extern REGARGFUN VOID _getservbyport();
extern REGARGFUN VOID _getprotobyname();
extern REGARGFUN VOID _getprotobynumber();

extern REGARGFUN VOID _Syslog();

extern REGARGFUN VOID _Dup2Socket();

f_void UserLibrary_funcTable[] = {
  (f_void)Null,		/* Open() */
  UL_Close,
  (f_void)Null,		/* Expunge() */
  (f_void)Null,		/* Reserved() */

  _socket,
  _bind,
  _listen,
  _accept,
  _connect,
  _sendto,
  _send,
  _recvfrom,
  _recv,
  _shutdown,
  _setsockopt,
  _getsockopt,
  _getsockname,
  _getpeername,

  _IoctlSocket,
  _CloseSocket,
  _WaitSelect,
  _SetSocketSignals,
  _SetDTableSize,
  _ObtainSocket,
  _ReleaseSocket,
  _ReleaseCopyOfSocket,
  _Errno,
  _SetErrnoPtr,

  _Inet_NtoA,
  _inet_addr,
  _Inet_LnaOf,
  _Inet_NetOf,
  _Inet_MakeAddr,
  _inet_network,

  _gethostbyname,
  _gethostbyaddr,
  _getnetbyname,
  _getnetbyaddr,
  _getservbyname,
  _getservbyport,
  _getprotobyname,
  _getprotobynumber,

  _Syslog,
  
  _Dup2Socket,
  (f_void)-1
};
