RCS_ID_C="$Id: amiga_log.c,v 1.34 1993/11/14 19:46:43 jraja Exp $";
/* 
 * Copyright (c) 1993 AmiTCP/IP Group, <amitcp-group@hut.fi>,
 *                    Helsinki University of Technology, Finland.
 *                    All rights reserved.
 *
 * HISTORY
 * $Log: amiga_log.c,v $
 * Revision 1.34  1993/11/14  19:46:43  jraja
 * Fixed off-by-one errors in month & day names.
 *
 * Revision 1.33  1993/11/06  23:51:22  ppessi
 * Changed log message format. Converted to use csprintf where possible.
 *
 * Revision 1.31  1993/10/07  22:41:34  ppessi
 * Added time to the log message.
 *
 * Revision 1.30  1993/08/06  08:58:37  jraja
 * Set required bsdsocket.library version to 2.
 *
 * Revision 1.29  1993/06/04  11:16:15  jraja
 * Fixes for first public release.
 *
 * Revision 1.28  1993/06/01  08:38:04  puhuri
 * Fixed declaration of logOpen. (missing static)
 *
 * Revision 1.27  1993/05/27  16:43:26  puhuri
 * Made better file opening (now tries everything).
 * Add few commets.
 *
 * Revision 1.26  1993/05/17  01:07:47  ppessi
 * Changed RCS version.
 *
 * Revision 1.25  1993/05/05  16:10:02  puhuri
 * Fixes for final demo.
 *
 * Revision 1.24  93/05/04  12:41:31  12:41:31  puhuri (Markus Peuhkuri)
 * Fixed defination of stuffchar(), perdence errors.
 * 
 * Revision 1.23  93/05/03  18:47:48  18:47:48  puhuri (Markus Peuhkuri)
 * Add check to GetLogMsg to avoid calling GetMsg with NULL pointer
 * (So we can call log before NETTRACE is activated, it just does not 
 * success) It also tries to open socket.library when it gots CTRL-F.
 * 
 * Revision 1.22  93/04/29  22:02:04  22:02:04  puhuri (Markus Peuhkuri)
 * Moved configuration structure log_cnf to amiga_log.h
 * 
 * Revision 1.21  93/04/28  19:24:30  19:24:30  puhuri (Markus Peuhkuri)
 * Fixed variables to give reliable configuration.
 * 
 * Revision 1.20  93/04/26  20:34:28  20:34:28  puhuri (Markus Peuhkuri)
 * Converts control-charactes to spaces, Add ability to modify
 * console and log filenames.
 * 
 * Revision 1.19  93/04/26  18:52:07  18:52:07  puhuri (Markus Peuhkuri)
 * Moved closelog() from subr_prf inside of log_deinit() as it was the
 * only place where it was calld. (Was for historial reasons)
 * Moved closing of Dos earlier in log_close to avoid problems in closing.
 * 
 * Revision 1.18  93/04/23  02:27:44  02:27:44  ppessi (Pekka Pessi)
 * Number and length of logging messages made configreable.
 * 
 * Revision 1.17  93/04/21  19:05:26  19:05:26  puhuri (Markus Peuhkuri)
 * Removed panic()'s if log file couldn't be opened/wroted. Add comments.
 * Now uses new structure of log_msg.
 * 
 * Revision 1.16  93/04/20  18:38:01  18:38:01  puhuri (Markus Peuhkuri)
 * Fixed prototype of log_poll, logmask was number of signal, not mask.
 * pointers to *file were automatic variables, now static.
 * 
 * Revision 1.15  93/04/17  17:56:29  17:56:29  puhuri (Markus Peuhkuri)
 * Fixed few trivial errors from previous version. 
 * (Should compile before ci)
 * 
 * Revision 1.14  93/04/17  17:16:51  17:16:51  puhuri (Markus Peuhkuri)
 * Moved REXX-stuff to NETTRACE-task (from amiga_main)
 * 
 * Revision 1.13  93/04/06  15:15:30  15:15:30  jraja (Jarno Tapio Rajahalme)
 * Changed spl function return value storage to spl_t,
 * changed bcopys and bzeros to aligned and/or const when possible,
 * added inclusion of conf.h to every .c file.
 * 
 * Revision 1.12  93/03/20  07:07:11  07:07:11  ppessi (Pekka Pessi)
 * Fixed memory leak caused by task_remove()
 * 
 * Revision 1.11  93/03/12  23:59:17  23:59:17  ppessi (Pekka Pessi)
 * Prototype fixes.
 * 
 * Revision 1.10  93/03/09  13:29:26  13:29:26  puhuri (Markus Peuhkuri)
 * Now stores information of failed log attempts using new function
 * GetLogMsg. (Increments the number of failed, which is then printed
 * (and zeroed) as logging task gets next message (ts handles it))
 * 
 * Revision 1.9  93/03/05  21:10:51  21:10:51  jraja (Jarno Tapio Rajahalme)
 * Fixed includes (again).
 * 
 * Revision 1.8  93/03/05  12:26:58  12:26:58  jraja (Jarno Tapio Rajahalme)
 * Added CloseLibrary() calls to close logDOSBase on exit.
 * 
 * Revision 1.7  93/03/05  03:25:45  03:25:45  ppessi (Pekka Pessi)
 * Compiles with SASC. Initial test version.
 * 
 * Revision 1.6  93/03/04  10:03:51  10:03:51  jraja (Jarno Tapio Rajahalme)
 * Fixed includes.
 * 
 * Revision 1.5  93/03/01  19:09:39  19:09:39  puhuri (Markus Peuhkuri)
 * Add variable.
 * 
 * Revision 1.4  93/02/26  19:42:46  19:42:46  puhuri (Markus Peuhkuri)
 * Modified to use MsgPort as list of free messages.
 * 
 * Revision 1.3  93/02/25  19:33:55  19:33:55  puhuri (Markus Peuhkuri)
 * Cleanup and consistency with other modules, removed amiga.lib stuff,
 * made compatible with SAS, log task has own DOSBase
 * 
 */

#include <conf.h>

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/time.h>

#include <exec/exec.h>
#include <dos/dos.h>
#include <dos/dostags.h>
#include <dos/dosextens.h>
#include <intuition/intuitionbase.h>

#include <kern/amiga_includes.h>
#include <sys/syslog.h>
#include <kern/amiga_log.h>
#include <kern/amiga_rexx.h>
#include <utility/date.h>

#if __SASC 
#include <proto/dos.h>
#include <proto/utility.h>
#elif __GNUC__
#define DOSBase logDOSBase
#define UtilityBase logUtilityBase
#include <inline/dos.h>
#undef DOSBase
#else 
#error Compiler not supported!
#endif

struct MsgPort *logPort,*logReplyPort;
struct DosLibrary *logDOSBase = NULL;
struct Library *logUtilityBase = NULL;
extern struct Task *AmiTCP_Task;
extern struct DosLibrary *DOSBase;
struct IntuitionBase *IntuitionBase = NULL;

extern void REGARGFUN stuffchar();
extern int logname_changed(void *p, LONG new);

static struct log_msg *log_poll(void);
static void log_task(void);
static void log_close(struct log_msg *msg);
static BPTR logOpen(STRPTR name);

struct log_msg *log_message;
static int GetLogMsgFail;


UBYTE consoledefname[] = _PATH_CON;
UBYTE logfiledefname[] = _PATH_LOG;
STRPTR logfilename = logfiledefname;
STRPTR consolename = consoledefname;


struct log_cnf log_cnf = { LOG_BUFS, LOG_BUF_LEN };

/*
 * Initialision function for log subsystem
 */

BOOL 
log_init(void)
{
  struct TagItem tags[] = { 
    NP_Entry, (LONG)&log_task,
    NP_Name, (LONG)LOG_TASK_NAME,
    NP_Priority, LOG_TASK_PRI,
    TAG_DONE,NULL };

  struct Message *msg;
  int i;
  char *buffer;
  ULONG sig;

  if (IntuitionBase && logReplyPort)
    return(TRUE);		/* We're allready open */

  if (!IntuitionBase &&		/* Open Intuition for EasyRequest */
     (IntuitionBase=(struct IntuitionBase *)
      OpenLibrary("intuition.library", 37L))==NULL)
    return(FALSE);
				/* Note that task which pops up EasyRequest
				 * in panic() is not nessissery the AmiTCP/IP
				 * but in versions <= 37 at least this
				 * does not matter for intuition
				 */

  /* Allocate buffers for log messages */
  if (log_message = AllocMem(sizeof(struct log_msg) * log_cnf.log_bufs,
			  MEMF_CLEAR|MEMF_PUBLIC))
    if (buffer=AllocMem(log_cnf.log_bufs * log_cnf.log_buf_len * sizeof(char),
		       MEMF_CLEAR|MEMF_PUBLIC)) {
      logPort=NULL;
      GetLogMsgFail=0;

      if (logReplyPort = CreateMsgPort()) {
	if (CreateNewProc(tags)) { /* Start NETTRACE task */
	  for (;;) {
				/* Wait signal for success of failure */
	    sig=Wait(1<<logReplyPort->mp_SigBit | SIGBREAKF_CTRL_F);
	    if (sig & SIGBREAKF_CTRL_F) /* Initializion failed maybe? */
	      if (logPort == (struct MsgPort *)-1) { /* YES, failed */
		logPort=NULL;
		goto fail;
	      }
	    if (msg=GetMsg(logReplyPort)) { /* Got message back? */
	      ReplyMsg(msg);
	      logReplyPort->mp_Flags = PA_IGNORE;
	      for (i = 0; i < log_cnf.log_bufs; i++) {
				/* Initialize messages */
		log_message[i].msg.mn_ReplyPort=logReplyPort;
		log_message[i].msg.mn_Length=sizeof(struct log_msg);
		log_message[i].level=0;
		log_message[i].string=buffer+i*log_cnf.log_buf_len;
		log_message[i].chars=0;
		PutMsg(logReplyPort,(struct Message *)&log_message[i]);
	      }
	      return(TRUE);	/* We're done */
	    }
	  }
	}
      }
    }
 fail:				/* Something went wrong */
  log_deinit();
  return(FALSE);
}

void log_deinit(void)
{
  struct log_msg *msg, *dump;

  if (logReplyPort) {		/* We have our port? */
    if (logPort) {		/* Logport exists? */
      logReplyPort->mp_Flags = PA_SIGNAL;

      while((msg = (struct log_msg *)GetMsg(logReplyPort)) == NULL) 
	/* Get a message */
	Wait(1<<logReplyPort->mp_SigBit);

				/* Initalize END_MESSAGE */
      msg->msg.mn_ReplyPort = logReplyPort;
      msg->msg.mn_Length = sizeof(struct log_msg);
      msg->level = LOG_CLOSE;

      PutMsg(logPort, (struct Message *)msg);
      
      for (;;) {
	dump = (struct log_msg *)GetMsg(logReplyPort);
	if (dump) {
	  if (dump->level == LOG_CLOSE)	/* Close message back */
	    break;		/* It was the last message */
	}	      
	else
	  Wait(1<<logReplyPort->mp_SigBit);
      }
    }

    while(GetMsg(logReplyPort))
      ;
    DeleteMsgPort(logReplyPort);
    logReplyPort = NULL;
    FreeMem(log_message[0].string, 
	    log_cnf.log_bufs * log_cnf.log_buf_len * sizeof(char));
    FreeMem(log_message, sizeof(struct log_msg) * log_cnf.log_bufs);
  }

  /* Close Intuition if nessassery */
  if (IntuitionBase) {
    CloseLibrary((struct Library *)IntuitionBase);
    IntuitionBase=NULL;
  }
}

/* A little stub for calling GetMsg w/ error reporting and cheking */
struct log_msg *GetLogMsg(struct MsgPort *port)
{
  struct Message *msg;

  if (port && (msg = GetMsg(port)))  /* Get a message */
    /* We should have a port, if not-> fail */
    return (struct log_msg *)msg; 

  ++GetLogMsgFail;		/* Increment number of failed messages */
  return NULL;
}

#define DOSBase logDOSBase	/* for SASC */
#define UtilityBase logUtilityBase

struct Library *SocketBase = NULL;
struct Task *Nettrace_Task = NULL;

/* Main loop for NETTRACE */
static 
void SAVEDS log_task()
{
  struct log_msg *initmsg=NULL;
  ULONG rexxmask = 0, logmask = 0, sigmask = 0;


  Nettrace_Task = FindTask(NULL); /* Store task pointer for AmiTCP */

  /* We need our own DosBase */
  if ((logDOSBase = (struct DosLibrary *)OpenLibrary(DOSNAME, 0L)) == NULL)
    goto fail;

  if ((logUtilityBase = OpenLibrary("utility.library", 37L)) == NULL)
    goto fail;

  /* Allocate message to reply startup */
  if ((initmsg = AllocMem(sizeof(struct log_msg), MEMF_CLEAR|MEMF_PUBLIC))
      == NULL)
    goto fail;

  /* Create our port for log messages */
  if ((logPort = CreateMsgPort()) == NULL)
    goto fail;

  logmask = 1<<logPort->mp_SigBit;

				/* Initialize rexx subsystem */
  if (!(rexxmask = rexx_init()))
    goto fail;

  initmsg->msg.mn_ReplyPort=logPort;
  initmsg->msg.mn_Length=sizeof(struct log_msg);
  
				/* Syncronize with AmiTCP/IP */
  PutMsg(logReplyPort,(struct Message *)initmsg);
  do {
    Wait(logmask);
  } while(initmsg!=(struct log_msg *)GetMsg(logPort));
  FreeMem(initmsg,sizeof(struct log_msg));
  initmsg=NULL;

  sigmask = logmask | rexxmask | SIGBREAKF_CTRL_F;

  for (;;) {			/* Main loop */
    ULONG sig;
    struct log_msg *msg;

    sig = Wait(sigmask);	/* Wait for signals */
    do {
				/* Signal for AmiTCP/IP: API ready */
      if ((sig & SIGBREAKF_CTRL_F) && (SocketBase == NULL)) {
	sig &= ~SIGBREAKF_CTRL_F;
	if (SocketBase = OpenLibrary("bsdsocket.library", 2L)) {
	  while(rexx_poll())	/* Check if we had got rexx messages */
	    ;
	  sigmask &= ~SIGBREAKF_CTRL_F;
	}
      }

      if (sig & logmask) {	/* Process log messages,
				 * handles all bending 
				 */
	if (msg = log_poll())	/* Got an LOG_CLOSE-message? */
	  return log_close(msg);
	sig &= ~logmask;
      }

      if (sig & rexxmask)	/* One rexx message at time */
	if (SocketBase) {
	  if (!rexx_poll()) 
	    sig &= ~rexxmask;
	} else
	  sig &= ~rexxmask;

      sig |= SetSignal(0L, sigmask) & sigmask; /* Signals left? */
    } while (sig);
  }

 fail:				
  /* Initializion Failed */
  if (initmsg)
    FreeMem(initmsg,sizeof(struct log_msg));
  log_close(NULL);

  logPort = (struct MsgPort *)-1;
  /* Inform AmiTCP that we failed */
  Signal(AmiTCP_Task, SIGBREAKF_CTRL_F); 
}

static BPTR confile=NULL;
static BPTR logfile=NULL;
static BOOL fileopenfail = FALSE, conopenfail = FALSE;
static BOOL filewritefail = FALSE, conwritefail = FALSE;

static char *months =
  "Jan\0Feb\0Mar\0Apr\0May\0Jun\0Jul\0Aug\0Sep\0Oct\0Nov\0Dec";

static char *wdays = 
  "Sun\0Mon\0Tue\0Wed\0Thu\0Fri\0Sat";

static char *levels = 
  "emerg\0"
  "alert\0"
  "crit \0"
  "err  \0"
  "warn \0"
  "note \0"
  "info \0"
  "debug";

/* 
 * Process all pending log messages 
 */
static
struct log_msg *log_poll()
{     
  struct log_msg *msg;
  ULONG where;
  LONG i;
  struct ClockData clockdata;

  /* The va_list argument for RawDoFmt() */
  struct {
    UBYTE *wday;
    UBYTE *month;
    WORD   mday;
    WORD   hour;
    WORD   min;
    WORD   sec;
#if HAVE_TIMEZONES
    UBYTE *tz;
#endif
    WORD   year;
    UBYTE *level;
  } tagdata;

  /* 28 for date, 14 for level */
# define LEVELBUF 28+14

  UBYTE buffer[LEVELBUF];
  static ULONG TotalFail;

  /* Process all messages */
  while (msg = (struct log_msg *)GetMsg(logPort)) { 
    struct CSource cs;

    where = TOLOG | ((msg->level != LOG_EMERG) ? TOCONS : 0);

    /* All except LOG_EMERG to both log and console */
    if (msg->level == LOG_CLOSE) {
      return (msg);
    }

    cs.CS_Buffer = buffer;
    cs.CS_Length = LEVELBUF;
    cs.CS_CurChr = 0;

    Amiga2Date(msg->time, &clockdata);

    tagdata.wday  = wdays + 4 * clockdata.wday;
    tagdata.month = months + 4 * (clockdata.month - 1);
    tagdata.mday  = clockdata.mday;
    tagdata.hour  = clockdata.hour;
    tagdata.min   = clockdata.min;
    tagdata.sec   = clockdata.sec;
#ifdef HAVE_TIMEZONES
    tagdata.tz    = "UCT";	/* Universal Coordinated Time */
#endif
    tagdata.year  = clockdata.year;
    tagdata.level = levels + 6 * 
      ((msg->level <= LOG_DEBUG) ? msg->level : LOG_DEBUG);

    vcsprintf(&cs, 
#ifdef HAVE_TIMEZONES
	      "%s %s %02d %02d:%02d:%02d %s %4d [%s]: ", 
#else
	      "%s %s %02d %02d:%02d:%02d %4d [%s]: ", 
#endif
	      (va_list)&tagdata);

    /* Remove last newline */
    if (msg->chars > 0 && msg->string[msg->chars - 1] == '\n') {
      msg->chars--;
    }

    /* Replace all control chars with space */
    for (i = 0; i < msg->chars; ++i) {
      if ((msg->string)[i] < ' ')
	(msg->string)[i] = ' ';
    }

    if (where & TOCONS)
      /* If console is not open, open it */
      if (confile || logname_changed(&consolename, (LONG)_PATH_CON)) {
	int error = 
	  FPuts(confile, buffer) == -1 ||
	    FWrite(confile, msg->string, msg->chars, 1) != 1 ||
	      FPutC(confile, '\n') == -1;
	Flush(confile);
	if (error && !conwritefail) {	/* To avoid loops */
	  conwritefail = TRUE;
	  log(LOG_ERR, "log: Write failed to console '%s'", consolename);
	}
      }

    if (where & TOLOG)
      /* If log file is not open, open it */
      if (logfile || logname_changed(&logfilename, (LONG)_PATH_LOG)) {
	int error = 
	  FPuts(logfile, buffer) == -1 ||
	    FWrite(logfile, msg->string, msg->chars, 1) != 1 ||
	      FPutC(logfile, '\n') == -1;
	Flush(logfile);
	if (error && !filewritefail) {	/* To avoid loops */
	  filewritefail = TRUE;
	  log(LOG_ERR, "log: Write failed to file '%s'", logfilename);
	}
      }

    ReplyMsg((struct Message *)msg);
    if (GetLogMsgFail != TotalFail) {
      int t = GetLogMsgFail;	/* Check if we have lost messages */

      log(LOG_WARNING,"%ld log messages lost (total %ld lost)\n",
	  t - TotalFail, TotalFail);
      TotalFail = t;
    }
  }
  return(NULL);
}  

/* Close logging subsystem */
static
void log_close(struct log_msg *msg)
{
  rexx_deinit();
  if (confile)
    Close(confile);
  if (logfile)
    Close(logfile);
  if (logUtilityBase) {
    CloseLibrary(logUtilityBase);
    logUtilityBase = NULL;
  }
  if (logDOSBase) {		/* DOS not needed below */
    CloseLibrary((struct Library *)logDOSBase);
    logDOSBase = NULL;
  }
  if (SocketBase) {
    CloseLibrary((struct Library *)SocketBase);
    SocketBase = NULL;
  }
  /*
   * Make sure that we get to end before task switch
   * and do not get messages from interrupts
   */
  Disable();

  if (logPort) {
    struct Message *m;

    while (m = GetMsg(logPort))	/* Check for messages and reply */
      ReplyMsg(m);

    DeleteMsgPort(logPort);	/* Delete port */
    logPort = NULL;
  }

  if (msg)
    ReplyMsg((struct Message *)msg);

  Nettrace_Task = NULL;
}

/*
 * Try first open w/ shared lock, then as old file and finaly new file
 */
static
BPTR logOpen(STRPTR name)
{
  BPTR file;

  if ((file = Open(name, MODE_READWRITE)) == NULL)
    if ((file = Open(name, MODE_OLDFILE)) == NULL)
      file = Open(name, MODE_NEWFILE);
 
  return file;
}
		  
int logname_changed(void *p, LONG new)
{
  UBYTE *newname = (UBYTE *)new;

  if (p == &logfilename) {	/* Is logname requested? */
    if (logfile)
      Close(logfile);
    if (logfile = logOpen(newname))
      return TRUE;
    log(LOG_ERR, "Opening of file '%s' failed\n", newname);
  } else if ( p == &consolename ) { /* Name of console */
    if (confile)
      Close(confile);
    if (confile = logOpen(newname))
      return TRUE;
    log(LOG_ERR, "Opening of console '%s' failed\n", newname);
  } else if ( p != NULL ) {	/* Some invalid pointer! */
    log(LOG_ERR, 
	"Invalid variable address in logname_changed() addr=0x%08lx\n", p);
    return FALSE;
  }

  if (!confile)			/* Here we try to open default files */
    if (!(confile = logOpen(consoledefname))) {
      if (! conopenfail) {
	conopenfail = TRUE;
	log(LOG_ERR,"Opening console '%s' failed", consoledefname);
      }
    } else {
      conopenfail = FALSE;
      consolename = consoledefname;
    }
  if (!logfile)
    if (!(logfile = logOpen(logfiledefname))) {
      if (! fileopenfail) {
	fileopenfail = TRUE;
	log(LOG_ERR,"Opening log '%s' failed", logfiledefname);
      }
    } else {
      fileopenfail = FALSE;
      logfilename = logfiledefname;
    }

  if ((p == &consoledefname && confile != NULL)
      || (p == &logfiledefname && logfile != NULL))
    return TRUE;


  return FALSE;
}
