RCS_ID_C = "$Id: net.c,v 6.2 93/10/15 02:59:10 ppessi Exp $";

/*
 * net.c - netfinger
 *
 * Copyright  1993 AmiTCP/IP Group, <AmiTCP-group@hut.fi>
 *                  Helsinki University of Technology, Finland.
 *
 * This program is derived from original work distributed in BSD Net 2.
 *
 * Created      : Fri Oct 15 01:29:07 1993 ppessi
 * Last modified: Fri Oct 15 01:45:10 1993 ppessi
 *
 * $Log:	net.c,v $
 * Revision 6.2  93/10/15  02:59:10  ppessi
 * Polished AmiTCP support
 * 
 */
 
/*
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Tony Nardo of the Johns Hopkins University/Applied Physics Lab.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

static char sccsid[] = "@(#)net.c	5.5 (Berkeley) 6/1/90";

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

void
netfinger(char *name)
{
  extern int lflag;
#if AMIGA
  char buf[1];
#else
  register FILE *fp;
#endif
  register int c, lastc;
  struct in_addr defaddr;
  struct hostent *hp, def;
  struct servent *sp;
  struct sockaddr_in sin;
  int s;
  char *alist[1], *host;
  u_long inet_addr();

  if (!(host = strchr(name, '@')))
    return;
  *host++ = NULL;
  if (!(hp = gethostbyname(host))) {
    defaddr.s_addr = inet_addr(host);
    if (defaddr.s_addr == -1) {
      (void)fprintf(stderr,
		    "finger: unknown host: %s\n", host);
      return;
    }
    def.h_name = host;
    def.h_addr_list = alist;
    def.h_addr = (char *)&defaddr;
    def.h_length = sizeof(struct in_addr);
    def.h_addrtype = AF_INET;
    def.h_aliases = 0;
    hp = &def;
  }
  if (!(sp = getservbyname("finger", "tcp"))) {
    (void)fprintf(stderr, "finger: tcp/finger: unknown service\n");
    return;
  }
  sin.sin_family = hp->h_addrtype;
  memcpy((char *)&sin.sin_addr, hp->h_addr, hp->h_length);
  sin.sin_port = sp->s_port;
  if ((s = socket(hp->h_addrtype, SOCK_STREAM, 0)) < 0) {
    perror("finger: socket");
    return;
  }

  /* have network connection; identify the host connected with */
  (void)printf("[%s]\n", hp->h_name);
  if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
    perror("finger: connect");
    (void)CloseSocket(s);
    return;
  }

  /* -l flag for remote fingerd  */
  if (lflag)
    (void)send(s, "/W ", 3, 0);
  /* send the name followed by <CR><LF> */
  (void)send(s, name, strlen(name), 0);
  (void)send(s, "\r\n", 2, 0);

  /*
   * Read from the remote system; once we're connected, we assume some
   * data.  If none arrives, we hang until the user interrupts.
   *
   * If we see a <CR> or a <CR> with the high bit set, treat it as
   * a newline; if followed by a newline character, only output one
   * newline.
   *
   * Otherwise, all high bits are stripped; if it isn't printable and
   * it isn't a space, we can simply set the 7th bit.  Every ASCII
   * character with bit 7 set is printable.
   */ 
#if AMIGA
  while (recv(s, buf, 1, 0) == 1) {
    c = buf[0]; 
    if (c == 0x0d) {
      c = '\n';
      lastc = '\r';
    } else {
      if (!isprint(c) && !isspace(c))
	c |= 0x40;
      if (lastc != '\r' || c != '\n')
	lastc = c;
      else {
	lastc = '\n';
	continue;
      }
    }
    putchar(c);
  }
  if (lastc != '\n')
    putchar('\n');
  CloseSocket(s);
#else
  if (fp = fdopen(s, "r"))
    while ((c = getc(fp)) != EOF) {
      c &= 0x7f;
      if (c == 0x0d) {
	c = '\n';
	lastc = '\r';
      } else {
	if (!isprint(c) && !isspace(c))
	  c |= 0x40;
	if (lastc != '\r' || c != '\n')
	  lastc = c;
	else {
	  lastc = '\n';
	  continue;
	}
      }
      putchar(c);
    }
  if (lastc != '\n')
    putchar('\n');
  (void)fclose(fp);
#endif
}

