/*
 *	File:					CheckDateTime.h
 *	Description:	Functions to handle and check date, time and calcfields.
 *
 *	(C) 1993, Ketil Hunn
 *
 */

#ifndef CHECKDATETIME_H
#define CHECKDATETIME_H

#define PREFIX_EXACT	0
#define PREFIX_BEFORE	1
#define PREFIX_AFTER	2

#include <time.h>
#include <string.h>
#include <dos.h>
#include "AdjustYear().h"

#define NONE	-1

// PROTOTYPES
int AdjustYear(int year);
int CountYears(struct EventNode *node);
int CountDays(struct EventNode *node);
int CheckPrefixDate(struct EventNode *node);
int CheckDate(struct EventNode *node);
int CheckPrefixTime(struct EventNode *node);
int CheckDateTime(struct EventNode *node);
char *CalcField(struct EventNode *node, char text[]);
char *CalcField(struct EventNode *node, char text[]);


long MakeTime(struct EventNode *node)
{
	struct tm input;

	input.tm_hour	=0;
	input.tm_min	=0;
	input.tm_sec	=0;
	input.tm_isdst=1;

	if(node==NULL)
	{
		char buf[8];

		getclk(buf);
		input.tm_mday	=buf[3];
		input.tm_mon	=buf[2]-1;
		input.tm_year	=buf[1]+80;
	}
	else
	{
		input.tm_mday	=node->day;
		input.tm_mon	=node->month-1;
		input.tm_year	=AdjustYear(node->year)-1900;
	}
	return (long)mktime(&input);
}

int CountYears(struct EventNode *node)
{
	char buf[8];
	int year, node_year;

	getclk(buf);
	year=buf[1]+1980;

	node_year=AdjustYear(node->year);
	return abs(year-node_year);
}

int CountDays(struct EventNode *node)
{
	struct DateStamp *date;
	time_t t;
	long	days1, days2;

	t=MakeTime(NULL);
	date=__timecvt(t);
	days1=date->ds_Days;

	t=MakeTime(node);
	date=__timecvt(t);
	days2=date->ds_Days;

	return abs(days2-days1);
}

int CheckPrefixDate(struct EventNode *node)
{
	char buf[8];
	short day=node->day, month=node->month, year=node->year;
	short now_day, now_month, now_year;

	getclk(buf);
	now_day		=buf[3];
	now_month	=buf[2];
	now_year	=buf[1]+1980;

	if(year)
		year=AdjustYear(year);

	switch(node->whendate)
	{
		case PREFIX_EXACT:
			return ((day ==now_day 			| day==0)		&
							(month ==now_month 	| month==0)	&
							(year ==now_year		| year==0));
			break;

		case PREFIX_BEFORE:
			if(now_year<year | year==0)
				return TRUE;
			else if(now_year==year)
				if(now_month<month | month==0)
					return TRUE;
				else if(now_month==month)
					if(now_day<day | day==0)
						return TRUE;
			break;

		case PREFIX_AFTER:
			if(now_year>year | year==0)
				return TRUE;
			else if (year==now_year)
				if(now_month>month | month==0)
					return TRUE;
				else if(now_month==month)
					if(now_day>day | day==0)
						return TRUE;
			break;
	}
	return FALSE;
}

int CheckDate(struct EventNode *node)
{
	if(CheckPrefixDate(node))
	{
		if(node->whendate==PREFIX_EXACT)
			return TRUE;
		else
		{
			int count=CountDays(node);

			if(node->repeat==0)
			{
				if(count<=node->days | node->days==0)
					return TRUE;
			}
			else
			{
				if(count%node->repeat==0)
					if(count<=node->days | node->days==0)
						return TRUE;
			}
		}
	}
	return FALSE;
}

int CheckPrefixTime(struct EventNode *node)
{
	char buf[8];
	short hour=node->hour, min=node->minutes;
	short now_hour, now_min;

	getclk(buf);
	now_hour	=buf[4];
	now_min		=buf[5];

	switch(node->whentime)
	{
		case PREFIX_EXACT:
			return ((hour	==now_hour	| hour==NONE) &&
							(min	==now_min		| min	==NONE));
			break;

		case PREFIX_BEFORE:
			if(now_hour<hour | hour==NONE)
				return TRUE;
			else if(now_hour==hour)
				if(now_min<min | min==NONE)
					return TRUE;
			break;

		case PREFIX_AFTER:
			if(now_hour>hour | hour==NONE)
				return TRUE;
			else if(now_hour==hour)
				if(now_min>=min | min==NONE)
					return TRUE;
			break;
	}
	return FALSE;
}

int CheckDateTime(struct EventNode *node)
{
	return (CheckDate(node) && CheckPrefixTime(node));
}

void StringToDate(char datestr[], struct EventNode *node)
{
	char tmp[3], *dummy, buf[8];

	if(strmid(datestr,tmp,1,2)==0)
	{
		node->day=strtol(tmp,&dummy,10);
		if(strmid(datestr,tmp,3,2)==0)
		{
			node->month=strtol(tmp,&dummy,10);
			if(strmid(datestr,tmp,5,2)==0)
				node->year=strtol(tmp,&dummy,10);
		}
	}
	getclk(buf);
	if(node->day==0)
		node->day=buf[3];
	if(node->month==0)
		node->month=buf[2];
	if(node->year==0)
		node->year=buf[1]+80;
}

long PosChr(char string[], int f, long pos)
{
	for(; string[pos]!=f & string[pos]!='\0'; ++pos)
		;
	if(string[pos]=='\0')
		return 0;
	return ++pos;
}

char *CalcField(struct EventNode *node, char text[])
{
	struct EventNode *calcdate;
	char calc[7], *before=text, *after;
	long pos=0;

	if(calcdate=AllocMem(sizeof(struct EventNode), MEMF_CLEAR))
	{
		while((pos=PosChr(text, '{', pos))!=0)
		{
			if(!strmid(text, calc, pos+1, 6))
			{
				StringToDate(calc, calcdate);
				after=text+pos+7;
				text[pos-1]='\0';
				sprintf(text, "%s%d%s", before,
																(node->calc==CALC_DAYS ? CountDays(calcdate) : CountYears(calcdate)),
																after);
			}
		}
		FreeMem(calcdate, sizeof(calcdate));
	}
	return text;
}
#endif
