/*
 *	Copyright (c) 1993 Michael D. Bayne.
 *	All rights reserved.
 *
 *	Please see the documentation accompanying the distribution for
 *      distribution and disclaimer information.
 */

#include <exec/types.h>
#include <exec/memory.h>

#include <dos/dos.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/alib_protos.h>

#include "Fireworks.h"
#include "/defs.h"
#include "/utility.h"

#define SCALE		100
#define ACCEL		4

#define	MAX_FIRE	200
#define	MAX_EXPLOSIONS	25

#define	NONE		0
#define	FIRE		1
#define	EXPLODING	2

struct Fire {
  BYTE type, time;
  ULONG x, y;
  LONG vx, vy, color;
};

struct mPrefObject {
  LONG Number;
  LONG Power;
  LONG Radius;
};

extern ULONG Depth, Mode;
extern UBYTE *prefData;
extern struct mPrefObject nP;
struct Fire *FireTable, *LastFire;
LONG MaxFires, NumFires, Power, Radius;
WORD Wid, Hei;

LONG CheckFire( struct Fire *f )
{
  if( f->x/SCALE >= Wid ) return( 1 );
  if( f->y/SCALE >= Hei ) return( 1 );
  return( 0 );
}

VOID IterateFire( struct RastPort *Rast )
{
  LONG	l;
  struct	Fire	*f, *tf;
  
  if(( NumFires < MaxFires )&&( RangeRand( 100 ) > 90 )) {
    for( tf = FireTable; tf->type; tf++ );
    if( tf != LastFire ) {
      tf->type = FIRE;
      tf->x = ( Wid / 4 + RangeRand( Wid / 2 )) * SCALE;
      tf->y = ( Hei - 5 ) * SCALE;
      tf->color = RangeRand(( 1 << Depth ) - 1 ) + 1;
      tf->vx = RangeRand( SCALE * Hei / 100 ) - SCALE;
      tf->vy = -1 * ( Power * SCALE * Hei / 400 );
      tf->time = RangeRand( 100 ) + 100;
      NumFires++;
    }
  }
  
  for( f = FireTable; f != LastFire; f++ ) {
    switch( f->type ) {
    case NONE:
      break;
    case FIRE:
      if( !f->time ) {
	for( l = 0, tf = FireTable; ( l < MAX_EXPLOSIONS )&&( tf != LastFire );
	    tf++ ) {
	  if( !tf->type ) {
	    CopyMem( f, tf, sizeof( struct Fire ));
	    tf->type = EXPLODING;
	    tf->vx = ( RangeRand( 100 ) - 50 ) * ( Radius * Wid / 1000 );
	    tf->vy = ( RangeRand( 100 ) - 50 ) * ( Radius * Hei / 1000 );
	    tf->time = RangeRand( 10 ) + 50;
	    l++;
	  }
	}
	NumFires--;
	f->type = NONE;
      } else {
	SetAPen( Rast, 0 );
	WritePixel( Rast, f->x/SCALE, f->y/SCALE );
	f->x += f->vx;
	f->y += f->vy;
	if( CheckFire( f )) {
	  f->type = NONE;
	  NumFires--;
	} else {
	  SetAPen( Rast, f->color );
	  WritePixel( Rast, f->x/SCALE, f->y/SCALE );
	  f->vy += ACCEL;
	  f->time--;
	}
      }
      break;
    case EXPLODING:
      SetAPen( Rast, 0 );
      WritePixel( Rast, f->x/SCALE, f->y/SCALE );
      if( !f->time ) f->type = NONE;
      else {
	f->x += f->vx;
	f->y += f->vy;
	if( CheckFire( f )) f->type = NONE;
	else {
	  SetAPen( Rast, f->color );
	  WritePixel( Rast, f->x/SCALE, f->y/SCALE );
	  f->vy += ACCEL;
	  f->time--;
	}
      }
      break;
    }
  }
}

VOID blank( VOID )
{
  struct mPrefObject *mP;
  struct Screen *Scr;
  if( FireworksWnd ) mP = &nP;
  else mP = ( struct mPrefObject * )prefData;
  
  MaxFires = mP->Number;
  Power = mP->Power;
  Radius = mP->Radius;
  
  FireTable = AllocVec( sizeof( struct Fire ) * ( MAX_FIRE + 1 ), MEMF_CLEAR );
  Scr = OpenScreenTags( NULL, SA_Depth, Depth, SA_Overscan, OSCAN_STANDARD,
		       SA_DisplayID, Mode, SA_Quiet, TRUE, SA_Behind, TRUE,
		       TAG_DONE );

  if( FireTable && Scr ) {
    LastFire = &( FireTable[MAX_FIRE] );
    
    if( Depth < 3 ) setCopperList( Scr->Height, 1, &( Scr->ViewPort ));
    
    Wid = Scr->Width;
    Hei = Scr->Height;
    
    SetRGB4(&( Scr->ViewPort ), 0, 0, 0, 0 );
    SetRGB4(&( Scr->ViewPort ), 1, 0x0F, 0x0F, 0x0F );
    SetRast(&( Scr->RastPort ), 0 );
    
    NumFires = 0;
    BlankMousePointer();
    ScreenToFront( Scr );
    
    while(!( SetSignal(0,0) & SIGBREAKF_CTRL_C )){
      WaitTOF();
      IterateFire(&( Scr->RastPort ));
    }
    
    if( FireTable ) FreeVec( FireTable );
    SetSignal( 0L, SIGBREAKF_CTRL_C );
    UnblankMousePointer();
    CloseScreen( Scr );
  }
}
