#include <exec/types.h>
#include <exec/memory.h>

#include <intuition/intuition.h>
#include <dos/dos.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/alib_protos.h>

#include "Plasma.h"
#include "/utility.h"

struct pPrefObject {
  LONG Param;
  LONG WrapAround;
};

extern struct pPrefObject nP;
extern UBYTE *prefData;
extern ULONG Depth, Mode;
UWORD Range[] = { 63, 31, 15, 7, 7, 3, 2, 2 }, Hei, Wid, Major, **Cell, Cl;
LONG WrapAround, Parameter;

void set( UWORD l, UWORD c, UWORD size, LONG value, struct RPort *R )
{
  UWORD rang;
  
  rang = Range[size];
  value = value + RangeRand( rang ) - ( rang + 1 ) / 2;
  if( WrapAround ) if(!( value % Cl )) value += Parameter;
  else if( value < 1 ) value = Parameter;
  Cell[c][l] = value;
  SetAPen( R, value );
  if(( c < Wid )&&( l < Hei )) WritePixel( R, c, l );
}

void grow( struct RPort *R, ULONG Parameter )
{
  UWORD l, c, i, step, nextStep, l1, l2, c1, c2;
  
  step = 256;
  for( i = 0; i < 8; i++ ) {
    nextStep = step / 2;
    for( l = 0; ( l < Major )&&( !( SetSignal( 0, 0 ) & SIGBREAKF_CTRL_C ));
	l += step ) {
      l1 = ( l + nextStep ) % Major;
      l2 = ( l + step ) % Major;
      for( c = 0; c < Major; c += step ) {
	c1 = ( c + nextStep ) % Major;
	c2 = ( c + step ) % Major;
	set( l, c1, i, ( Cell[l][c] + Cell[l][c2] + 1 )/2, R );
	set( l1, c, i, ( Cell[l][c] + Cell[l2][c] + 1 )/2, R );
	set( l1, c1, i, ( Cell[l][c] + Cell[l][c2] + Cell[l2][c] + 
			 Cell[l2][c2] + 2 )/4, R );
      }
    }
    step = nextStep;
  }
}

UWORD **FreeCells( UWORD **Cells )
{
  UWORD i = 0;
  
  if( Cells )
    for( i = 0; i < Major; i++ )
      if( Cells[i] ) FreeVec( Cells[i] );
  
  return( 0L );
}

UWORD **AllocCells( VOID )
{
  ULONG i, failed = 0;
  UWORD **Cell;
  
  if( Cell = AllocVec( Major * sizeof( UWORD * ), MEMF_CLEAR ))
    for( i = 0; i < Major; i ++ )
      if(!( Cell[i] = AllocVec( Major * sizeof( UWORD ), MEMF_CLEAR )))
	failed = 1;
  if( !failed ) return( Cell );
  return( FreeCells( Cell ));
}

VOID blank( VOID )
{
  struct Screen *Scr;
  LONG i;
  struct pPrefObject *pP;

  if( PlasmaWnd ) pP = &nP;
  else pP = ( struct pPrefObject * )prefData;
  
  if( Scr = OpenScreenTags( NULL, SA_Depth, Depth, SA_Overscan, OSCAN_STANDARD,
			   SA_DisplayID, Mode, SA_Quiet, TRUE, SA_Behind, TRUE,
			   TAG_DONE )) {
    Cl = 1 << Depth;
    Wid = Scr->Width;
    Hei = Scr->Height;
    Major = Wid > Hei ? Wid : Hei;
    WrapAround = pP->WrapAround;
    Parameter = pP->Param;

    RainbowPalette( Scr );
    BlankMousePointer();
    ScreenToFront( Scr );
    
    if( Cell = AllocCells() ) {
      while(!( SetSignal( 0L, 0L ) & SIGBREAKF_CTRL_C )) {
	
	SetRast(&( Scr->RastPort ), 0 );
	grow(&( Scr->RastPort ), pP->Param );
	
	for( i = 0; ( i < Wid * Hei / 100 )&&(!( SetSignal( 0L, 0L ) &
						SIGBREAKF_CTRL_C )); i++ ) {
	  WaitTOF();
	  WaitTOF();
	  RainbowPalette( Scr );
	}
      }
      SetSignal( 0L, SIGBREAKF_CTRL_C );
      FreeCells( Cell );
    }
    RainbowPalette( NULL );
    UnblankMousePointer();
    CloseScreen( Scr );
  }
}
