/*
 *      Copyright (c) 1993 Michael D. Bayne.
 *      All rights reserved.
 *
 *      Please see the documentation accompanying the distribution for
 *      distribution and disclaimer information.
 */

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/ports.h>

#include <dos/dos.h>
#include <dos/dostags.h>

#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#include <workbench/startup.h>
#include <workbench/workbench.h>
#include <libraries/reqtools.h>

#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/alib_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/utility_protos.h>
#include <clib/wb_protos.h>

#include <stdlib.h>

#include "Blanker.h"
#include "Garshneblanker_rev.h"
#include "defs.h"

STATIC const UBYTE VersTag[] = VERSTAG;

#include "protos/cxhand.h"
#include "protos/prefs.h"
#include "protos/winhand.h"
#include "protos/module.h"

#define OPENLIB( lb, lbn, v )\
  if(!( lb = OpenLibrary( lbn, v ))) exitBlanker( 1L )
#define OPENLIBCAST( lb, lbn, v )\
  if(!( lb = ( struct lb * )OpenLibrary( lbn, v ))) exitBlanker( 1L )

__far extern ULONG RangeSeed;
LONG blankSig, unblankSig, Blanking = FALSE;
ULONG tags[] = { RT_Window, 0L, RT_LockWindow, TRUE, TAG_END, 0L, TAG_END };
struct IntuitionBase *IntuitionBase = 0L;
struct GfxBase *GfxBase = 0L;
struct ReqToolsBase *ReqToolsBase = 0L;
struct Library *CxBase = 0L, *IconBase = 0L, *GadToolsBase = 0L;
struct Library *UtilityBase = 0L, *WorkbenchBase = 0L, *IFFParseBase = 0L;
struct Task *Task = 0L;
struct MsgPort *blankerPort = 0L, *modulePort = 0L, *appWinPort = 0L;
struct AppWindow *blankerAppWin = 0L;
struct RandomPrefsList *rPL = 0L;
struct Module *Module;
struct BlankerPrefs *Prefs;

VOID closeMainWindow( VOID )
{
  while( blankerAppWin ) if( RemoveAppWindow( blankerAppWin ))
    blankerAppWin = 0L;
  CloseBlankerWindow();
  tags[1] = 0L;
}

LONG openMainWindow( VOID )
{
  if( !BlankerWnd ) {
    if( SetupScreen()) return( 1 );
    OpenBlankerWindow();
    tags[1] = ( ULONG )BlankerWnd;
    CloseDownScreen();
    blankerAppWin = AddAppWindow( 1L, 0L, BlankerWnd, appWinPort, 0L );
  } else {
    ActivateWindow( BlankerWnd );
    WindowToFront( BlankerWnd );
  }
  
  if( !BlankerWnd ) return( QUIT );
  else ScreenToFront( BlankerWnd->WScreen );
  
  setPrefs( Prefs, Module, Blanker_CNT );
  return( OK );
}

VOID HandleBlankSignal( VOID )
{
  struct Module	*Temp;
  ULONG WasBlanking = Blanking;
  
  if(( Prefs->bp_Random )&&( Temp = NewModule( NULL, Prefs->bp_Dir,
					      GetRandomPref( rPL )))) {
    if( BlankerWnd ) {
      setPrefs( Prefs, Temp, GD_MODINFO );
      setPrefs( Prefs, Temp, GD_MDISPLAY );
    }
    MessageModule( Temp, BM_DOBLANK );
    if(( WasBlanking )&&( Module )) {
      Delay( 60 );
      Signal( Module->ml_Task, SIGBREAKF_CTRL_C );
    }
    FreeModule( Module );
    Module = Temp;
  } else if( !Blanking ) MessageModule( Module, BM_DOBLANK );

  SetSignal( 0L, cxBlankFlag );
}

VOID HandleAppMessage( VOID )
{
  struct AppMessage       *AppMsg;
  
  if( AppMsg = ( struct AppMessage * )GetMsg( appWinPort )) {
    struct Module *Temp = LoadArgListPrefs( AppMsg->am_ArgList,
					   AppMsg->am_NumArgs, Prefs );
    if( Temp ) {
      if( BlankerWnd ) {
	setPrefs( Prefs, Temp, GD_MODINFO );
	setPrefs( Prefs, Temp, GD_MDISPLAY );
      }
      FreeModule( Module );
      Module = Temp;
    }
    ReplyMsg(( struct Message * )AppMsg );
  }
}

VOID exitBlanker( LONG exitValue )
{
  FreeModule( Module );
  closeMainWindow();
  FreeGlobalToolPrefs( Prefs );
  
  if( rPL ) {
    FreeRPList( rPL );
    FreeVec( rPL );
  }
  
  if( appWinPort ) {
    struct Message *cleanMsg;
    
    while( cleanMsg = GetMsg( appWinPort )) ReplyMsg( cleanMsg );
    DeletePort( appWinPort );
  }
  
  if( blankerPort ) DeletePort( blankerPort );
  ShutdownCX();
  FreeSignal( blankSig );
  FreeSignal( unblankSig );
  
  if( IFFParseBase ) CloseLibrary( IFFParseBase );
  if( UtilityBase ) CloseLibrary( UtilityBase );
  if( WorkbenchBase ) CloseLibrary( WorkbenchBase );
  if( ReqToolsBase ) CloseLibrary(( struct Library * )ReqToolsBase );
  if( IconBase ) CloseLibrary( IconBase );
  if( CxBase ) CloseLibrary( CxBase );
  if( GfxBase ) CloseLibrary(( struct Library * )GfxBase );
  if( GadToolsBase ) CloseLibrary( GadToolsBase );
  if( IntuitionBase ) CloseLibrary(( struct Library * )IntuitionBase );
  
  exit( exitValue );
}

#ifdef DICE
VOID Kludge( VOID ) { _waitwbmsg(); }
VOID main( int argc, char *argv[] )
#else
VOID main( int argc, char *argv[] )
#endif
{
#ifdef DICE
  extern struct WBStartup *_WBMsg;
  struct WBStartup *WBSMsg = _WBMsg;
#else
  struct WBStartup *WBSMsg = 0L;
#endif

  ULONG sigs;
  LONG retval = 1;
  
  OPENLIBCAST( IntuitionBase, "intuition.library", 37L );
  OPENLIBCAST( GfxBase, "graphics.library", 37L );
  OPENLIB( GadToolsBase, "gadtools.library", 37L );
  OPENLIB( CxBase, "commodities.library", 37L );
  OPENLIB( IconBase, "icon.library", 36L );
  OPENLIBCAST( ReqToolsBase, REQTOOLSNAME, REQTOOLSVERSION );
  OPENLIB( WorkbenchBase, WORKBENCH_NAME, 37L );
  OPENLIB( UtilityBase, "utility.library", 37L );
  OPENLIB( IFFParseBase, "iffparse.library", 37L );
  
  if(!( rPL = AllocVec( sizeof( struct RandomPrefsList ), MEMF_CLEAR )))
    exitBlanker( 2L );
  
  RangeSeed = (ULONG)( Task = FindTask( 0L ));
  
  if(( blankSig = ( ULONG )AllocSignal( -1L )) == -1 ) exitBlanker( 3L );
  cxBlankFlag = 1L << blankSig;
  
  if(( unblankSig = ( ULONG )AllocSignal( -1L )) == -1 ) exitBlanker( 3L );
  cxUnblankFlag = 1L << unblankSig;
  
  if( CheckCX() == QUIT ) exitBlanker( 3L );
  
  if(!( blankerPort = CreatePort( "BlankerCC", 0 ))) exitBlanker( 4L );
  if(!( appWinPort = CreatePort( 0L, 0 ))) exitBlanker( 4L );
  
  Module = LoadDefaultPrefs( &Prefs );
#ifdef DICE
  if( WBSMsg &&( WBSMsg->sm_NumArgs > 1 ))
#else
    if(( !argc )&&( WBSMsg = ( struct WBStartup * )argv )&&
       ( WBSMsg->sm_NumArgs > 1 ))
#endif
      {
	struct Module *Temp = LoadArgListPrefs(&( WBSMsg->sm_ArgList[1] ),
					       WBSMsg->sm_NumArgs-1, Prefs );
	if( Temp ) {
	  FreeModule( Module );
	  Module = Temp;
	}
      }
  
  if( SetupCX() == QUIT ) exitBlanker( 5L );
  if( Prefs->bp_PopUp ) retval = openMainWindow();
  
  while( retval ) {
    sigs = Wait( cxBlankFlag | cxUnblankFlag | cxSigFlag | SIG_WIN |
		SIG_BREAK | SIG_APP );
    
    if( sigs & cxBlankFlag ) HandleBlankSignal();
    if(( sigs & cxUnblankFlag )&&( Blanking )) {
      Signal( Module->ml_Task, SIGBREAKF_CTRL_C );
      Blanking = FALSE;
    }
    if( sigs & cxSigFlag ) retval = HandleCxMess();
    if( sigs & SIG_WIN ) retval = HandleBlankerIDCMP();
    if( sigs & SIG_BREAK ) retval = 0L;
    if( sigs & SIG_APP ) HandleAppMessage();
    if( retval == CLOSEWIN ) closeMainWindow();
  }
  exitBlanker( 0L );
}
