/*
 *	Copyright (c) 1993 Michael D. Bayne.
 *	All rights reserved.
 *
 *      Please see the documentation accompanying the distribution for
 *      distribution and disclaimer information.
 */

#include <exec/types.h>
#include <exec/memory.h>

#include <dos/dos.h>
#include <dos/dostags.h>

#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/icon_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>

#include <graphics/displayinfo.h>
#include <libraries/commodities.h>

#include <intuition/intuitionbase.h>

#include <strings.h>

#include "defs.h"
#include "protos/main.h"
#include "protos/cxhand.h"
#include "protos/prefs.h"
#include "protos/winhand.h"
#include "blankers/utility.h"

VOID MessageModule( struct Module *Module, ULONG Type )
{
  if( Module ) {
    Module->ml_Message->bm_Type = Type;
    Module->ml_Message->bm_Blanking = &Blanking;

    PutMsg( Module->ml_Port, ( struct Message * )( Module->ml_Message ));
    WaitPort( blankerPort );
    GetMsg( blankerPort );

    if( Type == BM_DOBLANK ) Blanking = TRUE;
  } else if( Type == BM_DOBLANK ) {
    struct Screen *Scr = OpenScreenTags( 0L, SA_DisplayID, getTopScreenMode(),
					SA_Depth, 1, SA_Behind, 1, TAG_END );
    if( Scr ) {
      SetRGB4( &(Scr->ViewPort), 0, 0, 0, 0 );
      ScreenToFront( Scr );
      Blanking = TRUE;
      Wait( cxUnblankFlag );
      Blanking = FALSE;
      CloseScreen( Scr );
      SetSignal( 0L, cxUnblankFlag );
    }
  }
}

struct Module *FreeModule( struct Module *FreeMe )
{
  if( FreeMe ) {
    if( FreeMe->ml_Port ) MessageModule( FreeMe, BM_DOQUIT );
    if( FreeMe->ml_Message ) FreeVec( FreeMe->ml_Message );
    FreeVec( FreeMe );
  }
  return( 0L );
}

struct Module *NewModule( STRPTR Name, STRPTR Dir, STRPTR PrefFile )
{
  struct BlankerMessage	*Msg;
  struct Module		*New;
  BPTR			checkExists;
  ULONG			rv = 1;
  
  if(( !Dir )||( !Name && !PrefFile )) return( 0L );
  if(!( New = AllocVec( sizeof( struct Module ), MEMF_CLEAR ))) return( 0L );
  if(!( New->ml_Message = AllocVec( sizeof( struct BlankerMessage ),
				   MEMF_CLEAR ))) return( FreeModule( New ));
  
  New->ml_Message->bm_Mess.mn_ReplyPort = blankerPort;
  New->ml_Message->bm_Mess.mn_Length = sizeof( struct BlankerMessage );
  New->ml_Message->bm_Data = New->ml_Data;
  
  if( Name ) strcpy( New->ml_Name, Name );
  else rv = LoadModulePrefs( PrefFile, New );
  
  if( rv ) {
    strcpy( New->ml_IconPath, doPath( New->ml_Name, Dir ));
    
    if( checkExists = Open( "S:Blanker.log", MODE_OLDFILE )) {
      Seek( checkExists, 0, OFFSET_END );
      FPrintf( checkExists, "Attempting %s.\n", New->ml_IconPath );
      Close( checkExists );
    }
    
    if( checkExists = Open( New->ml_IconPath, MODE_OLDFILE )) {
      BPTR in = Open( "NIL:", MODE_OLDFILE );
      BPTR out = Open( "NIL:", MODE_OLDFILE );
      
      Close( checkExists );
      SystemTags( New->ml_IconPath, NP_FreeSeglist, TRUE, NP_Priority, -1,
		 SYS_Asynch, TRUE, SYS_Input, in, SYS_Output, out, TAG_END );
      WaitPort( blankerPort );
      
      if( Msg = ( struct BlankerMessage * )GetMsg( blankerPort )) {
	New->ml_Port = ( struct MsgPort * )Msg->bm_Mess.mn_ReplyPort;
	New->ml_Task = Msg->bm_Task;
	
	if( Msg->bm_Dep ) {
	  New->ml_Flags &= ~BS_ASKDEP;
	  New->ml_Message->bm_Dep = Msg->bm_Dep;
	} else {
	  New->ml_Flags |= BS_ASKDEP;
	  if ( Name ) New->ml_Message->bm_Dep = getTopScreenDepth();
	}
	
	if( Msg->bm_Mod != INVALID_ID ) {
	  New->ml_Flags &= ~BS_ASKSCR;
	  New->ml_Message->bm_Mod = Msg->bm_Mod;
	} else {
	  New->ml_Flags |= BS_ASKSCR;
	  if( Name ) New->ml_Message->bm_Mod = getTopScreenMode();
	}
	
	New->ml_Flags &= ~BS_NOPRFS;
	if( Msg->bm_Data ) {
	  if( Name ) CopyMem( Msg->bm_Data, New->ml_Data, 512 );
	} else New->ml_Flags |= BS_NOPRFS;
	
	strcpy( New->ml_InfoString, Msg->bm_Info );
	
	FreeMem( Msg, sizeof( struct BlankerMessage ));
	
	return( New );
      }
    }
  }
  
  return( FreeModule( New ));
}

ULONG getTopScreenMode( void )
{
  BPTR lock;
  struct	Screen	*pubScr;
  ULONG scrMode;
  
  lock = LockIBase( 0 );
  pubScr = IntuitionBase->FirstScreen;
  scrMode = GetVPModeID( &(pubScr->ViewPort));
  UnlockIBase( lock );
  return( scrMode );
}

ULONG getTopScreenDepth( void )
{
  BPTR lock;
  struct	Screen	*pubScr;
  struct	DrawInfo *dri;
  ULONG sDep = 0;
  
  lock = LockIBase( 0 );
  pubScr = IntuitionBase->FirstScreen;
  if( dri = GetScreenDrawInfo( pubScr )) {
    sDep = dri->dri_Depth;
    FreeScreenDrawInfo( pubScr, dri );
  }
  UnlockIBase( lock );
  return( sDep );
}
