/*
 *	Copyright (c) 1993 Michael D. Bayne.
 *	All rights reserved.
 *
 *	Please see the documentation accompanying the distribution for
 *      distribution and disclaimer information.
 */

#include <exec/types.h>
#include <exec/memory.h>

#include <dos/dos.h>
#include <workbench/startup.h>
#include <workbench/workbench.h>

#include <libraries/iffparse.h>
#include <libraries/reqtools.h>

#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/reqtools_protos.h>
#include <clib/graphics_protos.h>
#include <clib/iffparse_protos.h>
#include <clib/icon_protos.h>
#include <clib/wb_protos.h>
#include <clib/utility_protos.h>

#include <string.h>
#include <stdlib.h>

#include "defs.h"
#include "Blanker.h"

#define ID_PREF MAKE_ID('P','R','E','F')
#define ID_PRHD MAKE_ID('P','R','H','D')
#define ID_BLNK	MAKE_ID('B','L','N','K')

UWORD RangeRand( UWORD );

#include "protos/main.h"
#include "protos/winhand.h"
#include "protos/module.h"

UBYTE *doPath( UBYTE *file, UBYTE *dir )
{
  static UBYTE path[256];
  
  strcpy( path, dir );
  if( !AddPart( path, file, 256 )) return( 0L );
  return( path );
}

UBYTE *longToStr( LONG num )
{
  static	UBYTE	out[32];
  LONG	i = 32, j;
  
  while( num ) { out[--i] = num % 10; num /= 10; }
  for( j = 0; j < 32 - i; j++ ) out[j] = '0' + out[i+j];
  out[j] = 0;
  
  return( out );
}

VOID CreateIconWithToolTypes( UBYTE *path, UBYTE *toolTypes[],
			     UBYTE *defaultTool )
{
  struct DiskObject	*bDO;
  UBYTE			**oldToolTypes, *oldDefTool;
  
  if(!( bDO = GetDiskObject( path )))
    bDO = GetDiskObjectNew( "ENV:sys/def_pref" );
  
  if( bDO ) {
    oldToolTypes = bDO->do_ToolTypes;
    bDO->do_ToolTypes = toolTypes;
    
    if( defaultTool ) {
      oldDefTool = bDO->do_DefaultTool;
      bDO->do_DefaultTool = defaultTool;
    }
    
    PutDiskObject( path, bDO );
    
    bDO->do_ToolTypes = oldToolTypes;
    if( defaultTool ) bDO->do_DefaultTool = oldDefTool;
    
    FreeDiskObject( bDO );
  }
}

ULONG LoadModulePrefs( UBYTE *path, struct Module *Module )
{
  struct DiskObject	*bDO;
  struct IFFHandle	*bHandle;
  struct ContextNode	*top;
  UBYTE			*tooltype;
  
  if( Module ) {
    if(!( bHandle = AllocIFF())) return( NULL );
    if(!( bHandle->iff_Stream = (ULONG)Open( path, MODE_OLDFILE ))) goto FAIL;
    InitIFFasDOS( bHandle );
    if( OpenIFF( bHandle , IFFF_READ )) goto FAIL;
    
    for(;;) {
      if(( ParseIFF( bHandle, IFFPARSE_RAWSTEP ) == IFFERR_EOC )&&
	 ( top = CurrentChunk( bHandle ))&&( top->cn_Type == ID_PREF )&&
	 ( top->cn_ID == ID_BLNK )) {
	ReadChunkBytes( bHandle, &(Module->ml_Message->bm_Mod),
		       2 * sizeof( ULONG ));
	ReadChunkBytes( bHandle, Module->ml_Data, 512 );
	Module->ml_Flags |= BS_VALPRF;
	break;
      }
    }
    CloseIFF( bHandle );
    
  FAIL:
    if( bHandle->iff_Stream ) Close( bHandle->iff_Stream );
    FreeIFF( bHandle );
    
    if(( bDO = GetDiskObject( path ))&&
       ( tooltype = FindToolType( bDO->do_ToolTypes, "BLANKER" ))) {
      strcpy( Module->ml_Name, tooltype );
      FreeDiskObject( bDO );
      return( 1 );
    }
  }
  return( 0 );
}

VOID SaveModulePrefs( UBYTE *path, struct Module *Module )
{
  struct IFFHandle	*bHandle;
  UBYTE			prefHead[] = { 0, 0, 0, 0, 0, 0 };
  UBYTE			*newToolTypes[2], modNameToolType[128];
  
  if(!( bHandle = AllocIFF() )) goto FAIL;
  
  if( bHandle->iff_Stream = (ULONG)Open( path, MODE_NEWFILE )) {
    InitIFFasDOS( bHandle );
    
    if( OpenIFF( bHandle , IFFF_WRITE )) goto FAIL;
    
    PushChunk( bHandle, ID_PREF, ID_FORM, IFFSIZE_UNKNOWN );
    
    PushChunk( bHandle, ID_PREF, ID_PRHD, 6 );
    WriteChunkBytes( bHandle, prefHead, 6 );
    PopChunk( bHandle );
    
    PushChunk( bHandle, ID_PREF, ID_BLNK, 512 + 2 * sizeof( ULONG ));
    WriteChunkBytes( bHandle, &(Module->ml_Message->bm_Mod),
		    2 * sizeof( ULONG ));
    WriteChunkBytes( bHandle, Module->ml_Data, 512 );
    PopChunk( bHandle );
    
    PopChunk( bHandle );
    CloseIFF( bHandle );
  }
  
 FAIL:
  if( bHandle ) {
    if( bHandle->iff_Stream ) Close( bHandle->iff_Stream );
    FreeIFF( bHandle );
  }
  
  strcpy( modNameToolType, "BLANKER=" );
  strcat( modNameToolType, Module->ml_Name );
  newToolTypes[0] = modNameToolType;
  newToolTypes[1] = NULL;
  CreateIconWithToolTypes( path, newToolTypes, "Garshneblanker" );
}

VOID FreeGlobalToolPrefs( struct BlankerPrefs *bPO )
{
  if( bPO ) FreeVec( bPO );
}

struct BlankerPrefs *AllocGlobalToolPrefs( UBYTE *path )
{
  struct DiskObject	*bDO;
  struct BlankerPrefs	*bPO;
  UBYTE			*tooltype;
  
  if( bPO = AllocVec( sizeof( struct BlankerPrefs ), MEMF_CLEAR )) {
    if( bDO = GetDiskObject( path )) {
      if( tooltype = FindToolType( bDO->do_ToolTypes, "CX_PRIORITY" ))
	bPO->bp_Priority = atoi( tooltype );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "CX_POPUP" ))
	bPO->bp_PopUp = (ULONG)MatchToolValue( tooltype, "YES" );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "RANDOMPREFS" ))
	bPO->bp_Random = (ULONG)MatchToolValue( tooltype, "YES" );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "CX_POPKEY" ))
	strcpy( bPO->bp_PopKey, tooltype );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "BLANKKEY" ))
	strcpy( bPO->bp_BlankKey, tooltype );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "TIMEOUT" ))
	bPO->bp_Timeout = 10*atoi( tooltype );
      if( tooltype = FindToolType( bDO->do_ToolTypes, "BLANKERDIR" ))
	strcpy( bPO->bp_Dir, tooltype );
      FreeDiskObject( bDO );
    } else {
      bPO->bp_Priority = 1;
      bPO->bp_PopUp = TRUE;
      bPO->bp_Random = FALSE;
      strcpy( bPO->bp_PopKey, "Alt Help" );
      strcpy( bPO->bp_BlankKey, "Alt Delete" );
      bPO->bp_Timeout = 600;
      strcpy( bPO->bp_Dir, "Blankers" );
    }
  }
  
  return( bPO );
}

VOID WriteGlobalToolPrefs( UBYTE *path, struct BlankerPrefs *bPO )
{
#define	NUMTOOLTYPES		8
  LONG i;
  UBYTE *toolTypes[NUMTOOLTYPES+1];
  UBYTE *defToolTypes[] = { "DONOTWAIT", "CX_PRIORITY=", "CX_POPUP=",
			      "CX_POPKEY=", "BLANKKEY=", "TIMEOUT=",
			      "RANDOMPREFS=", "BLANKERDIR=" };
  
  for( i = 0; i < NUMTOOLTYPES; i++ )
    if( toolTypes[i] = AllocVec( 160, MEMF_CLEAR ))
      strcpy( toolTypes[i], defToolTypes[i] );
    else break;
  if( i < NUMTOOLTYPES ) while( --i > -1 ) FreeVec( toolTypes[i] );
  else {
    strcat( toolTypes[1], longToStr( bPO->bp_Priority ));
    strcat( toolTypes[2], bPO->bp_PopUp ? "YES" : "NO" );
    strcat( toolTypes[3], bPO->bp_PopKey );
    strcat( toolTypes[4], bPO->bp_BlankKey );
    strcat( toolTypes[5], longToStr( bPO->bp_Timeout/10 ));
    strcat( toolTypes[6], bPO->bp_Random ? "YES" : "NO" );
    strcat( toolTypes[7], bPO->bp_Dir );
    toolTypes[NUMTOOLTYPES] = NULL;
    
    CreateIconWithToolTypes( path, toolTypes, NULL );
    
    for( i = 0; i < NUMTOOLTYPES; i++ ) FreeVec( toolTypes[i] );
  }
}

struct Module *LoadArgListPrefs( struct WBArg *WBArgs, ULONG numArgs,
				struct BlankerPrefs *bPO )
{
  struct Module	*New;
  LONG		i;
  UBYTE		prefDir[108];
  
  for( i = 0; i < numArgs; i++, WBArgs++ )
    if( WBArgs->wa_Name && NameFromLock( WBArgs->wa_Lock, prefDir, 108 ))
      if( New = NewModule( NULL, bPO->bp_Dir,
			  doPath( WBArgs->wa_Name, prefDir )))
	break;
  return( New );
}

VOID SaveDefaultPrefs( struct BlankerPrefs *SavePrefs,
		      struct Module *SaveModule )
{
  WriteGlobalToolPrefs( "PROGDIR:Garshneblanker", SavePrefs );
  SaveModulePrefs( "ENV:Garshneblanker.prefs", SaveModule );
  SaveModulePrefs( "ENVARC:Garshneblanker.prefs", SaveModule );
}

struct Module *LoadDefaultPrefs( struct BlankerPrefs **bPO )
{
  struct Module *New;
  
  *bPO = AllocGlobalToolPrefs( "PROGDIR:Garshneblanker" );
  New = NewModule( NULL, ( *bPO )->bp_Dir, "ENV:Garshneblanker.prefs" );
  if( BlankerWnd ) setPrefs( *bPO, New, Blanker_CNT );
  
  return( New );
}

VOID FreeRPList( struct RandomPrefsList *rPL )
{
  struct RandomPrefs *Temp;
  
  rPL->rpl_Date.ds_Days = 0L;
  rPL->rpl_Date.ds_Minute = 0L;
  rPL->rpl_Date.ds_Tick = 0L;
  rPL->rpl_Number = 0;
  while( rPL->rpl_Head ) {
    Temp = rPL->rpl_Head;
    rPL->rpl_Head = Temp->rp_Next;
    FreeMem( Temp, sizeof( struct RandomPrefs ));
  }
}

VOID buildRandList( struct RandomPrefsList *rPrefs, UBYTE *dir )
{
  struct	FileInfoBlock *Blk;
  struct	RandomPrefs *Current;
  BPTR	lock;
  LONG	len;
  
  if(( Blk = AllocDosObject( DOS_FIB, 0L ))&&( lock = Lock( dir, ACCESS_READ ))
     &&( Examine( lock, Blk ))) {
    if( CompareDates( &rPrefs->rpl_Date, &Blk->fib_Date ) > 0 ) {
      FreeRPList( rPrefs );
      CopyMem( &Blk->fib_Date, &rPrefs->rpl_Date, sizeof( struct DateStamp ));
      while( ExNext( lock, Blk )) {
	if( Blk->fib_DirEntryType < 0 ) {
	  len = strlen( Blk->fib_FileName );
	  if( Stricmp( &(Blk->fib_FileName[len-5]), ".info" )&&
	     ( Current = AllocMem( sizeof( struct RandomPrefs ),
				  MEMF_CLEAR ))) {
	    strcpy( Current->rp_Name, Blk->fib_FileName );
	    rPrefs->rpl_Number += 1;
	    Current->rp_Next = rPrefs->rpl_Head;
	    rPrefs->rpl_Head = Current;
	  }
	}
      }
      if( IoErr() != ERROR_NO_MORE_ENTRIES ) rPrefs->rpl_Number = 0;
    }
  }
  if( Blk ) FreeDosObject( DOS_FIB, Blk );
  if( !rPrefs->rpl_Number ) FreeRPList( rPrefs );
}

UBYTE *GetRandomPref( struct RandomPrefsList *rPL )
{
  UWORD			i, num;
  struct RandomPrefs	*Current;
  UBYTE			*dir = "SYS:Prefs/Presets/Garshneblanker";
  BPTR			DebugFile;
  
  buildRandList( rPL, dir );
  
  if( rPL->rpl_Number ) {
    num = RangeRand(( UWORD )rPL->rpl_Number );
    for( i = 0, Current = rPL->rpl_Head;
	i < num; ++i, Current = Current->rp_Next );
#ifdef STDIO
    printf( "Chose %d: %s.\n", num, Current->rp_Name );
#endif
    if( DebugFile = Open( "S:Blanker.log", MODE_OLDFILE )) {
      Seek( DebugFile, 0, OFFSET_END );
      FPrintf( DebugFile, "Chose %s.\n", doPath( Current->rp_Name, dir ));
      Close( DebugFile );
    }
    return( doPath( Current->rp_Name, dir ));
  }
}
