/*
 *	Copyright (c) 1993 Michael D. Bayne.
 *	All rights reserved.
 *
 *	Please see the documentation accompanying the distribution for
 *      distribution and disclaimer information.
 */

#include <exec/types.h>
#include <exec/memory.h>

#include <intuition/intuition.h>
#include <intuition/gadgetclass.h>
#include <workbench/workbench.h>

#include <libraries/gadtools.h>
#include <libraries/reqtools.h>

#include <graphics/displayinfo.h>

#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/intuition_protos.h>
#include <clib/icon_protos.h>
#include <clib/gadtools_protos.h>
#include <clib/graphics_protos.h>
#include <clib/reqtools_protos.h>
#include <clib/alib_protos.h>

#include <strings.h>

#include "defs.h"
#include "Blanker.h"
#include "Garshneblanker_rev.h"

#include "protos/main.h"
#include "protos/cxhand.h"
#include "protos/module.h"
#include "protos/prefs.h"
#include "blankers/utility.h"

UBYTE	prefDir[108] = "SYS:Prefs/Presets/Garshneblanker";

VOID RenderModuleImage( struct Module *Module )
{
  ULONG			x, dx, y, dy;
  struct DiskObject	*bDO;
  struct Image		*Image;
  
  if( Module ) bDO = GetDiskObjectNew( Module->ml_IconPath );
  else bDO = GetDiskObjectNew( "PROGDIR:Garshneblanker" );
  
  if( bDO ) {
    Image = bDO->do_Gadget.GadgetRender;
    x = ( FontX * 6 + 4 ) / 8 + 3;
    y = ( FontY * 38 + 4 ) / 8 + 2;
    dx = ( FontX * 85 + 4 ) / 8 - 6;
    dy = ( FontY * 25 + 4 ) / 8 - 4;
    
    Image->LeftEdge = x + OffX + ( dx - Image->Width ) / 2;
    Image->TopEdge = y + OffY + ( dy - Image->Height ) / 2;
    EraseRect( BlankerWnd->RPort, x + OffX, y + OffY, x + dx + OffX,
	      y + dy + OffY );
    DrawImage( BlankerWnd->RPort, Image, 0, 0 );
    FreeDiskObject( bDO );
  }
}

void setPrefs( struct BlankerPrefs *cP, struct Module *cM, UBYTE gad )
{
  if(( gad == GD_TOUT )||( gad == Blanker_CNT ))
    GT_SetGadgetAttrs( BlankerGadgets[GD_TOUT], BlankerWnd, 0L, GTIN_Number,
		      (cP->bp_Timeout)/10, 0L );
  
  if(( gad == GD_PKEY )||( gad == Blanker_CNT ))
    GT_SetGadgetAttrs( BlankerGadgets[GD_PKEY], BlankerWnd, 0L, GTST_String,
		      cP->bp_PopKey, 0L );
  
  if(( gad == GD_BKEY )||( gad == Blanker_CNT ))
    GT_SetGadgetAttrs( BlankerGadgets[GD_BKEY], BlankerWnd, 0L, GTST_String,
		      cP->bp_BlankKey, 0L );
  
  if(( gad == GD_MODINFO )||( gad == Blanker_CNT )) {
    if( cM ) {
      GT_SetGadgetAttrs( BlankerGadgets[GD_BMOD], BlankerWnd, 0L, GA_Disabled,
			BS_NOPRFS & cM->ml_Flags, 0L );
      GT_SetGadgetAttrs( BlankerGadgets[GD_MODINFO], BlankerWnd, 0L, GTTX_Text,
			cM->ml_InfoString, 0L );
    } else {
      GT_SetGadgetAttrs( BlankerGadgets[GD_BMOD], BlankerWnd, 0L, GA_Disabled,
			TRUE, 0L );
      GT_SetGadgetAttrs( BlankerGadgets[GD_MODINFO], BlankerWnd, 0L, GTTX_Text,
			"No module loaded.", 0L );
    }
    RenderModuleImage( cM );
  }
  
  if(( gad == GD_MDISPLAY )||( gad == Blanker_CNT )) {
    struct NameInfo sInf;
    ULONG Mode = getTopScreenMode(), Depth = 1, AskScr = 0;
    UBYTE *ModeInfoStr;
    
    if( cM ) {
      Mode = cM->ml_Message->bm_Mod;
      Depth = cM->ml_Message->bm_Dep;
      AskScr = cM->ml_Flags & BS_ASKSCR;
    }
    
    if( GetDisplayInfoData( 0l, ( UBYTE * )&sInf, sizeof( struct NameInfo ),
			   DTAG_NAME, Mode ))
      ModeInfoStr = sInf.Name;
    else ModeInfoStr = "No Monitor Info";
    
    GT_SetGadgetAttrs( BlankerGadgets[GD_MDISPLAY], BlankerWnd, 0L, GTTX_Text,
		      ModeInfoStr, 0L );
    GT_SetGadgetAttrs( BlankerGadgets[GD_SDEP], BlankerWnd, 0L, GTNM_Number,
		      1 << Depth, 0L );
    GT_SetGadgetAttrs( BlankerGadgets[GD_SMODE], BlankerWnd, 0L, GA_Disabled,
		      !AskScr, 0L );
  }
  
  if( gad == Blanker_CNT ) {
    struct Menu *MenuStrip = BlankerWnd->MenuStrip;
    struct MenuItem *Item;
    
    ClearMenuStrip( BlankerWnd );
    
    Item = MenuStrip->NextMenu->NextMenu->FirstItem;
    if( Prefs->bp_Random ) Item->Flags |= CHECKED;
    else Item->Flags &= ~CHECKED;
    
    ResetMenuStrip( BlankerWnd, MenuStrip );
  }
}

int TOUTClicked( void )
{
  Prefs->bp_Timeout = 10*(((struct StringInfo*)(BlankerGadgets[GD_TOUT])->SpecialInfo)->LongInt );
  return( UpdateCX() );
}

int BKEYClicked( void )
{
  strcpy( Prefs->bp_BlankKey, ((struct StringInfo *)(BlankerGadgets[GD_BKEY])->SpecialInfo)->Buffer );
  return( UpdateCX() );
}

int PKEYClicked( void )
{
  strcpy( Prefs->bp_PopKey, ((struct StringInfo *)(BlankerGadgets[GD_PKEY])->SpecialInfo)->Buffer );
  return( UpdateCX() );
}

int HIDEClicked( void )
{
  return( CLOSEWIN );
}

int SAVEClicked( void )
{
  SaveDefaultPrefs( Prefs, Module );
  
  return( OK );
}

int QUITClicked( void )
{
  return( QUIT );
}

int SMODEClicked( void )
{
  struct rtScreenModeRequester	*smRequest;
  
  if( smRequest = rtAllocRequestA( RT_SCREENMODEREQ, 0l )) {
    rtChangeReqAttr( smRequest, RTSC_DisplayID, Module->ml_Message->bm_Mod, RTSC_DisplayDepth,
		    Module->ml_Message->bm_Dep, TAG_END );
    if( Module->ml_Flags & BS_ASKDEP ) {
      tags[4] = RTSC_Flags;
      tags[5] = SCREQF_DEPTHGAD;
      if( rtScreenModeRequestA( smRequest, "Screen Mode", BTAGS )) {
	Module->ml_Message->bm_Dep = smRequest->DisplayDepth;
	Module->ml_Message->bm_Mod = smRequest->DisplayID;
      }
      tags[4] = TAG_END;
    } else if( rtScreenModeRequestA( smRequest, "Screen Mode", BTAGS ))
      Module->ml_Message->bm_Mod = smRequest->DisplayID;
    rtFreeRequest( smRequest );
    setPrefs( Prefs, Module, GD_MDISPLAY );
  } else EZRQ( "Not enough memory to allocate\nscreen mode requester." );
  return( OK );
}

int BMODClicked( void )
{
  MessageModule( Module, BM_DOPREFS );
  
  return( OK );
}

int LMODClicked( void )
{
  static	UBYTE	Buffer[108] = "";
  
  struct rtFileRequester *fRequest;
  
  if( fRequest = rtAllocRequest( RT_FILEREQ, 0L )) {
    rtChangeReqAttr( fRequest, RTFI_Dir, Prefs->bp_Dir, TAG_END );
    tags[4] = RTFI_Flags;
    tags[5] = FREQF_PATGAD;
    if( rtFileRequestA( fRequest, Buffer, "Please choose blanker module...", BTAGS )) {
      strcpy( Prefs->bp_Dir, fRequest->Dir );
      FreeModule( Module );
      Module = NewModule( Buffer, Prefs->bp_Dir, NULL );
      setPrefs( Prefs, Module, GD_MODINFO );
      setPrefs( Prefs, Module, GD_MDISPLAY );
    }
    tags[4] = TAG_END;
    rtFreeRequest( fRequest );
  } else EZRQ( "Couldn't allocate file requester." );
  return( OK );
}

int BlankerOpen( void )
{
  struct rtFileRequester *fRequest;
  static	UBYTE	Buffer[108] = "";
  
  if( fRequest = rtAllocRequest( RT_FILEREQ, 0L )) {
    rtChangeReqAttr( fRequest, RTFI_Dir, prefDir, RTFI_MatchPat, "#?.pre", TAG_END );
    tags[4] = RTFI_Flags;
    tags[5] = FREQF_PATGAD;
    if( rtFileRequestA( fRequest, Buffer, "Please choose prefs file...", BTAGS )) {
      FreeModule( Module );
      Module = NewModule( NULL, Prefs->bp_Dir, doPath( Buffer, fRequest->Dir ));
      setPrefs( Prefs, Module, GD_MODINFO );
      setPrefs( Prefs, Module, GD_MDISPLAY );
    }
    tags[4] = TAG_END;
    rtFreeRequest( fRequest );
  } else EZRQ( "Couldn't allocate file requester." );
  return( OK );
}

int BlankerSaveAs( void )
{
  struct rtFileRequester *fRequest;
  static	UBYTE	Buffer[108];
  
  if( fRequest = rtAllocRequest( RT_FILEREQ, 0L )) {
    rtChangeReqAttr( fRequest, RTFI_Dir, prefDir, RTFI_MatchPat, "#?.pre", TAG_END );
    tags[4] = RTFI_Flags;
    tags[5] = FREQF_PATGAD;
    if( Module ) {
      strcpy( Buffer, Module->ml_Name );
      strcat( Buffer, ".pre" );
    }
    if( rtFileRequestA( fRequest, Buffer, "Please choose a save file...", BTAGS ))
      SaveModulePrefs( doPath( Buffer, fRequest->Dir ), Module );
    tags[4] = TAG_END;
    rtFreeRequest( fRequest );
  } else EZRQ( "Couldn't allocate file requester." );
  return( OK );
}

int BlankerAbout( void )
{
  EZRQ( VERS"\nby Michael D. Bayne\n\nbaynemd@nextwork.rose-hulman.edu" );
  return( OK );
}

int BlankerLoadMod( void )
{
  return( LMODClicked() );
}

int BlankerQuit( void )
{
  return( QUIT );
}

int BlankerLSaved( void )
{
  FreeModule( Module );
  FreeGlobalToolPrefs( Prefs );
  Module = LoadDefaultPrefs( &Prefs );
  return( OK );
}

int BlankerRandPrefs( void )
{
  Prefs->bp_Random = !Prefs->bp_Random;
  return( OK );
}

int BlankerCloseWindow( void )
{
  return( CLOSEWIN );
}

int BlankerVanillaKey( void )
{
  switch( BlankerMsg.Code ) {
  case 'c':
    return( SMODEClicked() );
  case 'r':
    return( BMODClicked() );
  case 'l':
    return( BlankerLoadMod());
  case 't':
    ActivateGadget( BlankerGadgets[GD_TOUT], BlankerWnd, 0l );
    return( OK );
  case 'b':
    ActivateGadget( BlankerGadgets[GD_BKEY], BlankerWnd, 0l );
    return( OK );
  case 'p':
    ActivateGadget( BlankerGadgets[GD_PKEY], BlankerWnd, 0l );
    return( OK );
  case 'h':
    return( HIDEClicked() );
  case 's':
    return( SAVEClicked() );
  case 'q':
    return( QUIT );
  default:
    return( OK );
  }
}
